@@ -33,6 +33,7 @@
 #include <pango/pango.h>
 
 #include <clutter/clutter-actor.h>
+#include <clutter/clutter-device-manager.h>
 #include <clutter/clutter-event.h>
 #include <clutter/clutter-feature.h>
 #include <clutter/clutter-stage.h>
@@ -64,49 +65,55 @@ struct _ClutterBackendClass
   GObjectClass parent_class;
 
   /* vfuncs */
-  gboolean            (* pre_parse)        (ClutterBackend  *backend,
-                                            GError         **error);
-  gboolean            (* post_parse)       (ClutterBackend  *backend,
-                                            GError         **error);
-  ClutterStageWindow *(* create_stage)     (ClutterBackend  *backend,
-                                            ClutterStage    *wrapper,
-                                            GError         **error);
-  void                (* init_events)      (ClutterBackend  *backend);
-  void                (* init_features)    (ClutterBackend  *backend);
-  void                (* add_options)      (ClutterBackend  *backend,
-                                            GOptionGroup    *group);
-  ClutterFeatureFlags (* get_features)     (ClutterBackend  *backend);
-  void                (* redraw)           (ClutterBackend  *backend,
-                                            ClutterStage    *stage);
-  gboolean            (* create_context)   (ClutterBackend  *backend,
-                                            GError         **error);
-  void                (* ensure_context)   (ClutterBackend  *backend,
-                                            ClutterStage    *stage);
+  gboolean              (* pre_parse)          (ClutterBackend  *backend,
+                                                GError         **error);
+  gboolean              (* post_parse)         (ClutterBackend  *backend,
+                                                GError         **error);
+  ClutterStageWindow *  (* create_stage)       (ClutterBackend  *backend,
+                                                ClutterStage    *wrapper,
+                                                GError         **error);
+  void                  (* init_events)        (ClutterBackend  *backend);
+  void                  (* init_features)      (ClutterBackend  *backend);
+  void                  (* add_options)        (ClutterBackend  *backend,
+                                                GOptionGroup    *group);
+  ClutterFeatureFlags   (* get_features)       (ClutterBackend  *backend);
+  void                  (* redraw)             (ClutterBackend  *backend,
+                                                ClutterStage    *stage);
+  gboolean              (* create_context)     (ClutterBackend  *backend,
+                                                GError         **error);
+  void                  (* ensure_context)     (ClutterBackend  *backend,
+                                                ClutterStage    *stage);
+  ClutterDeviceManager *(* get_device_manager) (ClutterBackend  *backend);
 
   /* signals */
   void (* resolution_changed) (ClutterBackend *backend);
   void (* font_changed)       (ClutterBackend *backend);
+  void (* settings_changed)   (ClutterBackend *backend);
 };
 
 GType clutter_backend_get_type    (void) G_GNUC_CONST;
 
 ClutterBackend *clutter_get_default_backend (void);
 
+#ifndef CLUTTER_DISABLE_DEPRECATED
 void                        clutter_backend_set_resolution            (ClutterBackend             *backend,
                                                                        gdouble                     dpi);
-gdouble                     clutter_backend_get_resolution            (ClutterBackend             *backend);
 void                        clutter_backend_set_double_click_time     (ClutterBackend             *backend,
                                                                        guint                       msec);
 guint                       clutter_backend_get_double_click_time     (ClutterBackend             *backend);
 void                        clutter_backend_set_double_click_distance (ClutterBackend             *backend,
                                                                        guint                       distance);
 guint                       clutter_backend_get_double_click_distance (ClutterBackend             *backend);
-void                        clutter_backend_set_font_options          (ClutterBackend             *backend,
-                                                                       const cairo_font_options_t *options);
-const cairo_font_options_t *clutter_backend_get_font_options          (ClutterBackend             *backend);
 void                        clutter_backend_set_font_name             (ClutterBackend             *backend,
                                                                        const gchar                *font_name);
 G_CONST_RETURN gchar *      clutter_backend_get_font_name             (ClutterBackend             *backend);
+#endif /* CLUTTER_DISABLE_DEPRECATED */
+
+gdouble                     clutter_backend_get_resolution            (ClutterBackend             *backend);
+
+void                        clutter_backend_set_font_options          (ClutterBackend             *backend,
+                                                                       const cairo_font_options_t *options);
+const cairo_font_options_t *clutter_backend_get_font_options          (ClutterBackend             *backend);
 
 G_END_DECLS
 
