@@ -9,7 +9,7 @@
  *              Iain Holmes <iain@o-hand.com>
  *              Neil Roberts <neil@linux.intel.com>
  *
- * Copyright (C) 2008  Intel Corporation.
+ * Copyright (C) 2008, 2009, 2010  Intel Corporation.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -22,9 +22,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
  */
 
 /**
@@ -302,8 +300,7 @@ clutter_cairo_texture_surface_resize_internal (ClutterCairoTexture *cairo)
 			       cairo,
                                clutter_cairo_texture_surface_destroy);
 
-  /* Create a blank Cogl texture
-   */
+  /* Create a blank Cogl texture */
   cogl_texture = cogl_texture_new_from_data (priv->width, priv->height,
                                              COGL_TEXTURE_NONE,
                                              CLUTTER_CAIRO_TEXTURE_PIXEL_FORMAT,
@@ -323,8 +320,8 @@ clutter_cairo_texture_notify (GObject    *object,
      that if both the width and height properties are set using a
      single call to g_object_set then the surface will only be resized
      once because the notifications will be frozen in between */
-  if (!strcmp ("surface-width", pspec->name)
-      || !strcmp ("surface-height", pspec->name))
+  if (strcmp ("surface-width", pspec->name) == 0 ||
+      strcmp ("surface-height", pspec->name) == 0)
     {
       ClutterCairoTexture *cairo = CLUTTER_CAIRO_TEXTURE (object);
 
@@ -370,6 +367,7 @@ clutter_cairo_texture_class_init (ClutterCairoTextureClass *klass)
 {
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
   ClutterActorClass *actor_class = CLUTTER_ACTOR_CLASS (klass);
+  GParamSpec *pspec;
 
   gobject_class->finalize     = clutter_cairo_texture_finalize;
   gobject_class->set_property = clutter_cairo_texture_set_property;
@@ -391,14 +389,15 @@ clutter_cairo_texture_class_init (ClutterCairoTextureClass *klass)
    *
    * Since: 1.0
    */
+  pspec = g_param_spec_uint ("surface-width",
+                             "Surface Width",
+                             "The width of the Cairo surface",
+                             0, G_MAXUINT,
+                             0,
+                             CLUTTER_PARAM_READWRITE);
   g_object_class_install_property (gobject_class,
                                    PROP_SURFACE_WIDTH,
-                                   g_param_spec_uint ("surface-width",
-                                                      "Surface-Width",
-                                                      "Surface Width",
-                                                      0, G_MAXUINT,
-                                                      0,
-                                                      CLUTTER_PARAM_READWRITE));
+                                   pspec);
   /**
    * ClutterCairoTexture:surface-height:
    *
@@ -407,14 +406,15 @@ clutter_cairo_texture_class_init (ClutterCairoTextureClass *klass)
    *
    * Since: 1.0
    */
+  pspec = g_param_spec_uint ("surface-height",
+                             "Surface-Height",
+                             "The height of the Cairo surface",
+                             0, G_MAXUINT,
+                             0,
+                             CLUTTER_PARAM_READWRITE);
   g_object_class_install_property (gobject_class,
                                    PROP_SURFACE_HEIGHT,
-                                   g_param_spec_uint ("surface-height",
-                                                      "Surface-Height",
-                                                      "Surface Height",
-                                                      0, G_MAXUINT,
-                                                      0,
-                                                      CLUTTER_PARAM_READWRITE));
+                                   pspec);
 }
 
 static void
@@ -430,6 +430,14 @@ clutter_cairo_texture_init (ClutterCairoTexture *self)
    * match that format
    */
   priv->format = CAIRO_FORMAT_ARGB32;
+
+  /* the Cairo surface is responsible for driving the size of
+   * the texture; if we let sync_size to its default of TRUE,
+   * the Texture will try to queue a relayout every time we
+   * change the size of the Cairo surface - which is not what
+   * we want
+   */
+  clutter_texture_set_sync_size (CLUTTER_TEXTURE (self), FALSE);
 }
 
 /**
