@@ -17,9 +17,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
  *
  * Original author:
  *
@@ -209,7 +207,7 @@ clutter_script_flags_from_string (GType        type,
   const gchar *flag;
 
   g_return_val_if_fail (G_TYPE_IS_FLAGS (type), 0);
-  g_return_val_if_fail (string != 0, 0);
+  g_return_val_if_fail (string != NULL, 0);
 
   ret = TRUE;
   
@@ -489,8 +487,8 @@ clutter_script_parse_color (ClutterScript *script,
   return FALSE;
 }
 
-static const gchar *
-get_id_from_node (JsonNode *node)
+const gchar *
+_clutter_script_get_id_from_node (JsonNode *node)
 {
   JsonObject *object;
 
@@ -533,7 +531,7 @@ parse_children (ObjectInfo *oinfo,
       JsonNode *child = json_array_get_element (array, i);
       const gchar *id;
 
-      id = get_id_from_node (child);
+      id = _clutter_script_get_id_from_node (child);
       if (id)
         retval = g_list_prepend (retval, g_strdup (id));
     }
@@ -674,7 +672,7 @@ parse_behaviours (ObjectInfo *oinfo,
       JsonNode *child = json_array_get_element (array, i);
       const gchar *id;
 
-      id = get_id_from_node (child);
+      id = _clutter_script_get_id_from_node (child);
       if (id)
         retval = g_list_prepend (retval, g_strdup (id));
     }
@@ -913,18 +911,27 @@ clutter_script_parser_object_end (JsonParser *json_parser,
   const gchar *id;
   GList *members, *l;
 
+  /* if the object definition does not have an 'id' field we'll
+   * fake one for it...
+   */
   if (!json_object_has_member (object, "id"))
     {
       gchar *fake;
 
+      /* ... unless it doesn't even have a type - in which case
+       * it is an internal object definition and we're not
+       * supposed to touch it
+       */
       if (!json_object_has_member (object, "type"))
         return;
 
       fake = _clutter_script_generate_fake_id (script);
+      json_object_set_string_member (object, "id", fake);
 
-      val = json_node_new (JSON_NODE_VALUE);
-      json_node_set_string (val, fake);
-      json_object_set_member (object, "id", val);
+      CLUTTER_NOTE (SCRIPT,
+                    "Adding fake id '%s' to object of type '%s'",
+                    json_object_get_string_member (object, "id"),
+                    json_object_get_string_member (object, "type"));
 
       g_free (fake);
     }
@@ -940,9 +947,10 @@ clutter_script_parser_object_end (JsonParser *json_parser,
     }
 
   id = json_object_get_string_member (object, "id");
+  CLUTTER_NOTE (SCRIPT, "Getting object info for object '%s'", id);
 
   oinfo = _clutter_script_get_object_info (script, id);
-  if (G_LIKELY (!oinfo))
+  if (oinfo == NULL)
     {
       const gchar *class_name;
 
@@ -960,6 +968,9 @@ clutter_script_parser_object_end (JsonParser *json_parser,
           type_func = json_object_get_string_member (object, "type_func");
           oinfo->type_func = g_strdup (type_func);
 
+          /* remove the type_func member; we don't want it to
+           * pollute the object members
+           */
           json_object_remove_member (object, "type_func");
         }
     }
@@ -1010,18 +1021,37 @@ clutter_script_parser_object_end (JsonParser *json_parser,
       PropertyInfo *pinfo;
       JsonNode *node;
 
+      CLUTTER_NOTE (SCRIPT, "Object '%s' member '%s'",
+                    oinfo->id,
+                    name);
+
       /* we have already parsed these */
       if (strcmp (name, "id") == 0 || strcmp (name, "type") == 0)
         continue;
 
       node = json_object_get_member (object, name);
 
+      /* this should not really happen; we're getting a list of
+       * member names, and if one does not map a real member
+       * value then it's likely that something has gone wrong
+       */
+      if (G_UNLIKELY (node == NULL))
+        {
+          CLUTTER_NOTE (SCRIPT,
+                        "Empty node for member '%s' of object '%s' (type: %s)",
+                        name,
+                        oinfo->id,
+                        oinfo->class_name);
+          continue;
+        }
+
       pinfo = g_slice_new (PropertyInfo);
 
       pinfo->name = g_strdup (name);
       pinfo->node = json_node_copy (node);
       pinfo->pspec = NULL;
       pinfo->is_child = g_str_has_prefix (name, "child::") ? TRUE : FALSE;
+      pinfo->is_layout = g_str_has_prefix (name, "layout::") ? TRUE : FALSE;
 
       oinfo->properties = g_list_prepend (oinfo->properties, pinfo);
     }
@@ -1085,7 +1115,7 @@ clutter_script_parse_node (ClutterScript *script,
                * definitions are parsed leaf-first we are guaranteed
                * to have a defined object at this point
                */
-              id = get_id_from_node (node);
+              id = _clutter_script_get_id_from_node (node);
               if (id == NULL || *id == '\0')
                 return FALSE;
 
@@ -1393,9 +1423,11 @@ clutter_script_translate_parameters (ClutterScript  *script,
       GParameter param = { NULL };
       gboolean res = FALSE;
 
-      if (pinfo->is_child)
+      if (pinfo->is_child || pinfo->is_layout)
         {
-          CLUTTER_NOTE (SCRIPT, "Child property '%s' ignored", pinfo->name);
+          CLUTTER_NOTE (SCRIPT, "Skipping %s property '%s'",
+                        pinfo->is_child ? "child" : "layout",
+                        pinfo->name);
           unparsed = g_list_prepend (unparsed, pinfo);
           continue;
         }
@@ -1501,6 +1533,112 @@ clutter_script_construct_parameters (ClutterScript  *script,
   return unparsed;
 }
 
+static void
+apply_layout_properties (ClutterScript    *script,
+                         ClutterContainer *container,
+                         ClutterActor     *actor,
+                         ObjectInfo       *oinfo)
+{
+  ClutterScriptable *scriptable = NULL;
+  ClutterScriptableIface *iface = NULL;
+  gboolean set_custom_property = FALSE;
+  gboolean parse_custom_node = FALSE;
+  GList *l, *unresolved, *properties;
+  ClutterLayoutManager *manager;
+  GType meta_type;
+
+  manager = g_object_get_data (G_OBJECT (container), "clutter-layout-manager");
+  if (manager == NULL)
+    return;
+
+  meta_type = _clutter_layout_manager_get_child_meta_type (manager);
+  if (meta_type == G_TYPE_INVALID)
+    return;
+
+  CLUTTER_NOTE (SCRIPT, "Layout manager of type '%s' with meta type '%s'",
+                G_OBJECT_TYPE_NAME (manager),
+                g_type_name (meta_type));
+
+  /* shortcut, to avoid typechecking every time */
+  if (CLUTTER_IS_SCRIPTABLE (manager))
+    {
+      scriptable = CLUTTER_SCRIPTABLE (manager);
+      iface = CLUTTER_SCRIPTABLE_GET_IFACE (scriptable);
+
+      parse_custom_node = iface->parse_custom_node != NULL ? TRUE : FALSE;
+      set_custom_property = iface->set_custom_property != NULL ? TRUE : FALSE;
+    }
+
+  properties = oinfo->properties;
+  oinfo->properties = NULL;
+
+  unresolved = NULL;
+  for (l = properties; l != NULL; l = l->next)
+    {
+      PropertyInfo *pinfo = l->data;
+      GValue value = { 0, };
+      gboolean res = FALSE;
+      const gchar *name;
+
+      if (!pinfo->is_layout)
+        {
+          unresolved = g_list_prepend (unresolved, pinfo);
+          continue;
+        }
+
+      name = pinfo->name + strlen ("layout::");
+
+      pinfo->pspec =
+        clutter_layout_manager_find_child_property (manager, name);
+
+      if (pinfo->pspec != NULL)
+        g_param_spec_ref (pinfo->pspec);
+
+      CLUTTER_NOTE (SCRIPT, "Parsing %s layout property (id:%s)",
+                    pinfo->pspec != NULL ? "regular" : "custom",
+                    name);
+
+      if (parse_custom_node)
+        res = iface->parse_custom_node (scriptable, script, &value,
+                                        name,
+                                        pinfo->node);
+
+      if (!res)
+        res = clutter_script_parse_node (script, &value,
+                                         name,
+                                         pinfo->node,
+                                         pinfo->pspec);
+
+      if (!res)
+        {
+          CLUTTER_NOTE (SCRIPT, "Layout property '%s' ignored", name);
+          unresolved = g_list_prepend (unresolved, pinfo);
+          continue;
+        }
+
+      CLUTTER_NOTE (SCRIPT,
+                    "Setting %s layout property '%s' (type:%s) to "
+                    "object '%s' (id:%s)",
+                    set_custom_property ? "custom" : "regular",
+                    name,
+                    g_type_name (G_VALUE_TYPE (&value)),
+                    g_type_name (oinfo->gtype),
+                    oinfo->id);
+
+      clutter_layout_manager_child_set_property (manager, container, actor,
+                                                 name,
+                                                 &value);
+
+      g_value_unset (&value);
+
+      property_info_free (pinfo);
+    }
+
+  g_list_free (properties);
+
+  oinfo->properties = unresolved;
+}
+
 static void
 apply_child_properties (ClutterScript    *script,
                         ClutterContainer *container,
@@ -1669,15 +1807,23 @@ add_children (ClutterScript *script,
           continue;
         }
 
+      if (!CLUTTER_IS_ACTOR (object))
+        {
+          g_warning ("The object definition '%s' (type: %s) is not "
+                     "an actor, but it is referenced in the 'children' "
+                     "member of the container '%s' (type: %s); skipping.",
+                     child_info->id,
+                     g_type_name (child_info->gtype),
+                     oinfo->id,
+                     g_type_name (oinfo->gtype));
+          continue;
+        }
+
       CLUTTER_NOTE (SCRIPT, "Adding children '%s' to actor of type '%s'",
                     name,
                     g_type_name (G_OBJECT_TYPE (container)));
 
       clutter_container_add_actor (container, CLUTTER_ACTOR (object));
-
-      apply_child_properties (script,
-                              container, CLUTTER_ACTOR (object),
-                              child_info);
     }
 
   g_list_foreach (oinfo->children, (GFunc) g_free, NULL);
@@ -1686,26 +1832,6 @@ add_children (ClutterScript *script,
   oinfo->children = unresolved;
 }
 
-/* top-level classes: these classes are the roots of the
- * hiearchy; some of them must be unreferenced, whilst
- * others are owned by other instances
- */
-static const struct
-{
-  const gchar *type_name;
-  guint is_toplevel : 1;
-} clutter_toplevels[] = {
-  { "ClutterActor",          FALSE },
-  { "ClutterAlpha",          FALSE },
-  { "ClutterBehaviour",      TRUE  },
-  { "ClutterEffectTemplate", TRUE  },
-  { "ClutterModel",          TRUE  },
-  { "ClutterScore",          TRUE  },
-  { "ClutterTimeline",       TRUE  }
-};
-
-static guint n_clutter_toplevels = G_N_ELEMENTS (clutter_toplevels);
-
 static inline void
 _clutter_script_check_unresolved (ClutterScript *script,
                                   ObjectInfo    *oinfo)
@@ -1716,6 +1842,49 @@ _clutter_script_check_unresolved (ClutterScript *script,
   if (oinfo->behaviours != NULL && CLUTTER_IS_ACTOR (oinfo->object))
     apply_behaviours (script, oinfo);
 
+  /* this is a bit *eugh*, but it allows us to effectively make sure
+   * that child and layout properties are parsed and applied to the
+   * right child
+   */
+  if (oinfo->properties != NULL && CLUTTER_IS_ACTOR (oinfo->object))
+    {
+      ClutterActor *parent;
+
+      parent = clutter_actor_get_parent (CLUTTER_ACTOR (oinfo->object));
+      if (parent != NULL && CLUTTER_IS_CONTAINER (parent))
+        {
+          ClutterContainer *container = CLUTTER_CONTAINER (parent);
+          ClutterActor *actor = CLUTTER_ACTOR (oinfo->object);
+          GList *children, *l;
+
+          children = clutter_container_get_children (container);
+
+          for (l = children; l != NULL; l = l->next)
+            {
+              GObject *child = l->data;
+              ObjectInfo *child_info;
+              const gchar *id;
+
+              id = clutter_get_script_id (child);
+              if (id == NULL || *id == '\0')
+                continue;
+
+              child_info = _clutter_script_get_object_info (script, id);
+              if (child_info == NULL)
+                continue;
+
+              apply_child_properties (script, container,
+                                      actor,
+                                      child_info);
+              apply_layout_properties (script, container,
+                                       actor,
+                                       child_info);
+            }
+
+          g_list_free (children);
+        }
+    }
+
   if (oinfo->properties || oinfo->children || oinfo->behaviours)
     oinfo->has_unresolved = TRUE;
   else
@@ -1812,18 +1981,10 @@ _clutter_script_construct_object (ClutterScript *script,
       if (G_UNLIKELY (oinfo->gtype == G_TYPE_INVALID))
         return;
 
-      for (i = 0; i < n_clutter_toplevels; i++)
-        {
-          const gchar *t_name = clutter_toplevels[i].type_name;
-          GType t_type;
-
-          t_type = clutter_script_get_type_from_name (script, t_name);
-          if (g_type_is_a (oinfo->gtype, t_type))
-            {
-              oinfo->is_toplevel = clutter_toplevels[i].is_toplevel;
-              break;
-            }
-        }
+      oinfo->is_toplevel =
+        g_type_is_a (oinfo->gtype, G_TYPE_INITIALLY_UNOWNED)
+          ? FALSE
+          : TRUE;
     }
 
   if (oinfo->gtype == CLUTTER_TYPE_STAGE && oinfo->is_stage_default)
