@@ -18,9 +18,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
+ *
+ *
  */
 
 #ifndef __CLUTTER_SCRIPT_PRIVATE_H__
@@ -80,6 +80,7 @@ typedef struct {
   GParamSpec *pspec;
 
   guint is_child : 1;
+  guint is_layout : 1;
 } PropertyInfo;
 
 typedef struct {
@@ -148,6 +149,8 @@ guint _clutter_script_get_last_merge_id (ClutterScript *script);
 void _clutter_script_add_object_info (ClutterScript *script,
                                       ObjectInfo    *oinfo);
 
+const gchar *_clutter_script_get_id_from_node (JsonNode *node);
+
 G_END_DECLS
 
 #endif /* __CLUTTER_SCRIPT_PRIVATE_H__ */
