@@ -25,8 +25,11 @@
  * SECTION:clutter-state
  * @short_description: State machine with animated transitions
  *
- * #ClutterState controls the tweening of properties on multiple
- * actors between a set of named states.
+ * #ClutterState is an object controlling the tweening of properties on
+ * multiple actors between a set of named states. #ClutterStateKey<!-- -->s
+ * define how the properties are animated. If the source_state_name for a key
+ * is NULL it is used for transition to the target state unless a specific key
+ * exists for transitioning from the current state to the requested state.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -114,6 +117,7 @@ typedef struct _ClutterStateKey
 enum
 {
   PROP_0,
+  PROP_DURATION,
   PROP_TARGET_STATE
 };
 
@@ -168,6 +172,9 @@ sort_props_func (gconstpointer a,
   return pa->object - pb->object;
 }
 
+static State * clutter_state_get_state (ClutterState *state,
+                                        const gchar  *state_name,
+                                        gboolean      force_creation);
 static void object_disappeared (gpointer data,
                                 GObject *where_the_object_was);
 
@@ -254,8 +261,7 @@ clutter_state_remove_key_internal (ClutterState *this,
   property_name = g_intern_string (property_name);
 
   if (source_state_name)
-    source_state = g_hash_table_lookup (this->priv->states,
-                                        source_state_name);
+    source_state = clutter_state_get_state (this, source_state_name, FALSE);
 
   if (target_state_name != NULL)
     state_list = g_list_append (NULL, (gpointer) target_state_name);
@@ -265,13 +271,13 @@ clutter_state_remove_key_internal (ClutterState *this,
   for (s = state_list; s != NULL; s = s->next)
     {
       State *target_state;
-      target_state = g_hash_table_lookup (this->priv->states, s->data);
+      target_state = clutter_state_get_state (this, s->data, FALSE);
 
       if (target_state)
         {
           GList *k;
 
-          for (k = target_state->keys; k; k = k->next)
+          for (k = target_state->keys; k != NULL; k = k?k->next:NULL)
             {
               ClutterStateKey *key = k->data;
 
@@ -314,7 +320,7 @@ state_free (gpointer data)
        state->keys = g_list_remove (state->keys, state->keys->data))
     clutter_state_key_free (state->keys->data);
 
-  g_array_unref (state->animators);
+  g_array_free (state->animators, TRUE);
   g_hash_table_destroy (state->durations);
   g_free (state);
 }
@@ -396,6 +402,7 @@ clutter_state_new_frame (ClutterTimeline *timeline,
         {
           if (key->source_state != NULL &&
               key->source_state->name != NULL &&
+              priv->source_state_name != NULL && 
               g_str_equal (priv->source_state_name, key->source_state->name))
             {
               found_specific = TRUE;
@@ -437,7 +444,7 @@ clutter_state_new_frame (ClutterTimeline *timeline,
  * @target_state_name: the state to transition to
  * @animate: whether we should animate the transition or not
  *
- * Change the current state of #ClutterState to @target__name
+ * Change the current state of #ClutterState to @target_state_name
  *
  * If @animate is %FALSE, the state transition will happen immediately;
  * otherwise, the state transition will be animated over the duration
@@ -463,13 +470,8 @@ clutter_state_change (ClutterState *state,
 
   priv = state->priv;
 
-  if (target_state_name == NULL)
-    target_state_name = "default";
-
-  target_state_name = g_intern_string (target_state_name);
-
-  if (priv->target_state_name == NULL)
-    priv->target_state_name = g_intern_static_string ("default");
+  if (target_state_name != NULL)
+    target_state_name = g_intern_string (target_state_name);
 
   if (target_state_name == priv->target_state_name)
     {
@@ -493,7 +495,7 @@ clutter_state_change (ClutterState *state,
                                          priv->target_state_name);
   clutter_timeline_set_duration (priv->timeline, duration);
 
-  new_state = g_hash_table_lookup (priv->states, target_state_name);
+  new_state = clutter_state_get_state (state, target_state_name, FALSE);
   if (new_state == NULL)
     {
       g_warning ("State '%s' not found", target_state_name);
@@ -623,14 +625,15 @@ get_property_from_object (GObject     *gobject,
  * For instance, the code below:
  *
  * |[
- *   clutter_state_set (state, "default", "hover",
+ *   clutter_state_set (state, NULL, "hover",
  *                      button, "opacity", 255, CLUTTER_LINEAR,
  *                      button, "scale-x", 1.2, CLUTTER_EASE_OUT_CUBIC,
  *                      button, "scale-y", 1.2, CLUTTER_EASE_OUT_CUBIC,
  *                      NULL);
  * ]|
  *
- * will create a transition between the "default" and "hover" state; the
+ * will create a transition from any state (a @source_state_name of NULL is
+ * treated as a wildcard) and a state named "hover"; the
  * <emphasis>button</emphasis> object will have the #ClutterActor:opacity
  * property animated to a value of 255 using %CLUTTER_LINEAR as the animation
  * mode, and the #ClutterActor:scale-x and #ClutterActor:scale-y properties
@@ -805,16 +808,13 @@ clutter_state_get_state (ClutterState *state,
 
   if (state_name == NULL)
     {
-      if (force_creation)
-        state_name = g_intern_static_string ("default");
-      else
-        return NULL;
+      return NULL;
     }
   else
     state_name = g_intern_string (state_name);
 
   retval = g_hash_table_lookup (priv->states, state_name);
-  if (retval == NULL)
+  if (retval == NULL && force_creation)
     {
       retval = state_new (state, state_name);
       g_hash_table_insert (priv->states, (gpointer) state_name, retval);
@@ -826,7 +826,9 @@ clutter_state_get_state (ClutterState *state,
 /**
  * clutter_state_set_key:
  * @state: a #ClutterState instance.
- * @source_state_name: the source transition to specify transition for
+ * @source_state_name: the source transition to specify transition for or NULL
+ *   to specify the default fallback when a more specific source_state doesn't
+ *   exist.
  * @target_state_name: the name of the transition to set a key value for.
  * @object: the #GObject to set a key for
  * @property_name: the property to set a key for
@@ -873,7 +875,7 @@ clutter_state_set_key (ClutterState  *state,
   if (pspec == NULL)
     return state;
 
-  source_state = clutter_state_get_state (state, source_state_name, FALSE);
+  source_state = clutter_state_get_state (state, source_state_name, TRUE);
   target_state = clutter_state_get_state (state, target_state_name, TRUE);
 
   property_name = g_intern_string (property_name);
@@ -961,14 +963,13 @@ clutter_state_get_keys (ClutterState *state,
     state_list = clutter_state_get_states (state);
 
   if (source_state_name)
-    source_state = g_hash_table_lookup (state->priv->states,
-                                        source_state_name);
+    source_state = clutter_state_get_state (state, source_state_name, FALSE);
 
   for (s = state_list; s != NULL; s = s->next)
     {
       State *target_state;
 
-      target_state = g_hash_table_lookup (state->priv->states, s->data);
+      target_state = clutter_state_get_state (state, s->data, FALSE);
       if (target_state != NULL)
         {
           GList *k;
@@ -1060,6 +1061,9 @@ clutter_state_set_property (GObject      *object,
       case PROP_TARGET_STATE:
         clutter_state_change (state, g_value_get_string (value), TRUE);
         break;
+      case PROP_DURATION:
+        state->priv->duration = g_value_get_uint (value);
+        break;
       default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
     }
@@ -1078,6 +1082,9 @@ clutter_state_get_property (GObject    *object,
         g_value_set_string (value,
                             clutter_state_get_target_state (state));
         break;
+      case PROP_DURATION:
+        g_value_set_uint (value, state->priv->duration);
+        break;
       default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
     }
@@ -1110,7 +1117,7 @@ clutter_state_class_init (ClutterStateClass *klass)
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (ClutterStateClass, completed),
                   NULL, NULL,
-                  clutter_marshal_VOID__VOID,
+                  _clutter_marshal_VOID__VOID,
                   G_TYPE_NONE, 0);
 
   /**
@@ -1124,9 +1131,22 @@ clutter_state_class_init (ClutterStateClass *klass)
   pspec = g_param_spec_string ("target-state",
                                "Target State",
                                "Currently set state",
-                               "default",
+                               NULL,
                                CLUTTER_PARAM_READWRITE);
   g_object_class_install_property (gobject_class, PROP_TARGET_STATE, pspec);
+
+  /**
+   * ClutterState:duration:
+   *
+   * Default duration used if an duration has not been specified for a specific
+   * source/target state pair. The values is in milliseconds.
+   */
+  pspec = g_param_spec_uint ("duration",
+                             "Duration",
+                             "Default transition duration",
+                             0, 86400000, 1000,
+                             CLUTTER_PARAM_READWRITE);
+  g_object_class_install_property (gobject_class, PROP_DURATION, pspec);
 }
 
 static void
@@ -1182,13 +1202,12 @@ clutter_state_get_animator (ClutterState *state,
   g_return_val_if_fail (CLUTTER_IS_STATE (state), NULL);
 
   source_state_name = g_intern_string (source_state_name);
-  if (source_state_name == g_intern_static_string ("default") ||
-      source_state_name == g_intern_static_string (""))
+  if (source_state_name == g_intern_static_string (""))
     source_state_name = NULL;
 
   target_state_name = g_intern_string (target_state_name);
 
-  target_state = g_hash_table_lookup (state->priv->states, target_state_name);
+  target_state = clutter_state_get_state (state, target_state_name, FALSE);
   if (target_state == NULL)
     return NULL;
   
@@ -1239,7 +1258,7 @@ clutter_state_set_animator (ClutterState    *state,
   source_state_name = g_intern_string (source_state_name);
   target_state_name = g_intern_string (target_state_name);
 
-  target_state = g_hash_table_lookup (state->priv->states, target_state_name);
+  target_state = clutter_state_get_state (state, target_state_name, FALSE);
   if (target_state == NULL)
     return;
   
@@ -1512,13 +1531,11 @@ clutter_state_set_duration (ClutterState *state,
   g_return_if_fail (CLUTTER_IS_STATE (state));
 
   source_state_name = g_intern_string (source_state_name);
-  if (source_state_name == g_intern_static_string ("default") ||
-      source_state_name == g_intern_static_string (""))
+  if (source_state_name == g_intern_static_string (""))
     source_state_name = NULL;
 
   target_state_name = g_intern_string (target_state_name);
-  if (target_state_name == g_intern_static_string ("default") ||
-      target_state_name == g_intern_static_string (""))
+  if (target_state_name == g_intern_static_string (""))
     target_state_name = NULL;
 
   if (target_state_name == NULL)
@@ -1527,7 +1544,7 @@ clutter_state_set_duration (ClutterState *state,
       return;
     }
 
-  target_state = g_hash_table_lookup (state->priv->states, target_state_name);
+  target_state = clutter_state_get_state (state, target_state_name, FALSE);
   if (target_state != NULL)
     {
       if (source_state_name != NULL)
@@ -1570,20 +1587,17 @@ clutter_state_get_duration (ClutterState *state,
   g_return_val_if_fail (CLUTTER_IS_STATE (state), 0);
 
   source_state_name = g_intern_string (source_state_name);
-  if (source_state_name == g_intern_static_string ("default") ||
-      source_state_name == g_intern_static_string (""))
+  if (source_state_name == g_intern_static_string (""))
     source_state_name = NULL;
 
   target_state_name = g_intern_string (target_state_name);
-  if (target_state_name == g_intern_static_string ("default") ||
-      target_state_name == g_intern_static_string (""))
+  if (target_state_name == g_intern_static_string (""))
     target_state_name = NULL;
 
   if (target_state_name == NULL)
     return state->priv->duration;
 
-  target_state = g_hash_table_lookup (state->priv->states,
-                                      target_state_name);
+  target_state = clutter_state_get_state (state, target_state_name, FALSE);
   if (target_state != NULL)
     {
       if (source_state_name)
@@ -1680,7 +1694,7 @@ parse_state_transition (JsonArray *array,
     }
 
   source_name = json_object_get_string_member (object, "source");
-  source_state = clutter_state_get_state (clos->state, source_name, FALSE);
+  source_state = clutter_state_get_state (clos->state, source_name, TRUE);
 
   target_name = json_object_get_string_member (object, "target");
   target_state = clutter_state_get_state (clos->state, target_name, TRUE);
