@@ -72,7 +72,7 @@
 
 static void _cogl_atlas_texture_free (CoglAtlasTexture *sub_tex);
 
-COGL_HANDLE_DEFINE (AtlasTexture, atlas_texture);
+COGL_TEXTURE_DEFINE (AtlasTexture, atlas_texture);
 
 static const CoglTextureVtable cogl_atlas_texture_vtable;
 
@@ -146,7 +146,9 @@ _cogl_atlas_texture_blit_begin (CoglAtlasTextureBlitData *data,
           data->fbo = 0;
         }
 
-      GE( glBindTexture (data->dst_gl_target, dst_gl_texture) );
+      _cogl_bind_gl_texture_transient (data->dst_gl_target,
+                                       dst_gl_texture,
+                                       FALSE);
     }
 
   if (data->fbo)
@@ -229,13 +231,18 @@ _cogl_atlas_texture_foreach_sub_texture_in_region (
 }
 
 static void
-_cogl_atlas_texture_set_wrap_mode_parameter (CoglTexture *tex,
-                                             GLenum wrap_mode)
+_cogl_atlas_texture_set_wrap_mode_parameters (CoglTexture *tex,
+                                              GLenum wrap_mode_s,
+                                              GLenum wrap_mode_t,
+                                              GLenum wrap_mode_r)
 {
   CoglAtlasTexture *atlas_tex = COGL_ATLAS_TEXTURE (tex);
 
   /* Forward on to the sub texture */
-  _cogl_texture_set_wrap_mode_parameter (atlas_tex->sub_texture, wrap_mode);
+  _cogl_texture_set_wrap_mode_parameters (atlas_tex->sub_texture,
+                                          wrap_mode_s,
+                                          wrap_mode_t,
+                                          wrap_mode_r);
 }
 
 static void
@@ -268,6 +275,9 @@ _cogl_atlas_texture_free (CoglAtlasTexture *atlas_tex)
   _cogl_atlas_texture_remove_from_atlas (atlas_tex);
 
   cogl_handle_unref (atlas_tex->sub_texture);
+
+  /* Chain up */
+  _cogl_texture_free (COGL_TEXTURE (atlas_tex));
 }
 
 static int
@@ -355,12 +365,19 @@ _cogl_atlas_texture_migrate_out_of_atlas (CoglAtlasTexture *atlas_tex)
 
       COGL_NOTE (ATLAS, "Migrating texture out of the atlas");
 
-      /* Although this texture is being moved out of the atlas, we
-         don't need to flush the journal here because the contents of
-         the old image will still be left in the same position in the
-         atlas. The only way this would get overwritten would be if a
-         new texture was added, but in that case we flush the journal
-         anyway */
+      /* We don't know if any materials may currently be referenced in
+       * the journal that depend on the current underlying GL texture
+       * storage so we flush the journal before migrating.
+       *
+       * We are assuming that texture atlas migration never happens
+       * during a flush so we don't have to consider recursion here.
+       */
+      _cogl_journal_flush ();
+
+      /* Notify cogl-material.c that the texture's underlying GL texture
+       * storage is changing so it knows it may need to bind a new texture
+       * if the CoglTexture is reused with the same texture unit. */
+      _cogl_material_texture_storage_change_notify (atlas_tex);
 
       cogl_handle_unref (atlas_tex->sub_texture);
 
@@ -390,16 +407,17 @@ _cogl_atlas_texture_migrate_out_of_atlas (CoglAtlasTexture *atlas_tex)
 }
 
 static void
-_cogl_atlas_texture_ensure_mipmaps (CoglTexture *tex)
+_cogl_atlas_texture_pre_paint (CoglTexture *tex, CoglTexturePrePaintFlags flags)
 {
   CoglAtlasTexture *atlas_tex = COGL_ATLAS_TEXTURE (tex);
 
-  /* Mipmaps do not work well with the current atlas so instead we'll
-     just migrate the texture out and use a regular texture */
-  _cogl_atlas_texture_migrate_out_of_atlas (atlas_tex);
+  if ((flags & COGL_TEXTURE_NEEDS_MIPMAP))
+    /* Mipmaps do not work well with the current atlas so instead
+       we'll just migrate the texture out and use a regular texture */
+    _cogl_atlas_texture_migrate_out_of_atlas (atlas_tex);
 
   /* Forward on to the sub texture */
-  _cogl_texture_ensure_mipmaps (atlas_tex->sub_texture);
+  _cogl_texture_pre_paint (atlas_tex->sub_texture, flags);
 }
 
 static void
@@ -661,10 +679,24 @@ _cogl_atlas_texture_migrate (unsigned int                    n_textures,
   unsigned int i;
   CoglAtlasTextureBlitData blit_data;
 
+  /* We don't know if any materials may currently be referenced in the
+   * journal that depend on the current underlying GL texture storage
+   * so we flush the journal before migrating.
+   *
+   * We are assuming that texture atlas migration never happens during
+   * a flush so we don't have to consider recursion here.
+   */
+  _cogl_journal_flush ();
+
   _cogl_atlas_texture_blit_begin (&blit_data, new_texture, old_texture);
 
   for (i = 0; i < n_textures; i++)
     {
+      /* Notify cogl-material.c that the texture's underlying GL texture
+       * storage is changing so it knows it may need to bind a new texture
+       * if the CoglTexture is reused with the same texture unit. */
+      _cogl_material_texture_storage_change_notify (textures[i].texture);
+
       /* Skip the texture that is being added because it doesn't contain
          any data yet */
       if (textures[i].texture != skip_texture)
@@ -977,22 +1009,6 @@ _cogl_atlas_texture_new_from_bitmap (CoglHandle       bmp_handle,
       return COGL_INVALID_HANDLE;
     }
 
-  /* We need to flush the journal when adding a new atlas texture for
-     two reasons:
-
-     1) If the atlas gets reorganized then any existing textures may
-        be moved so the coordinates in journal will be wrong.
-
-     2) We don't want to flush before migrating a texture out of the
-        atlas because we will potentially already be in the middle of
-        a flush to cause the migration. Therefore we rely on the old
-        contents of the texture still remaining in the atlas so that
-        any existing journal entries will still be valid. However once
-        the texture is migrated out the space would be free to be
-        reclaimed by this new texture. That won't matter if we flush
-        now */
-  _cogl_journal_flush ();
-
   /* We need to allocate the texture now because we need the pointer
      to set as the data for the rectangle in the atlas */
   atlas_tex = g_new (CoglAtlasTexture, 1);
@@ -1066,11 +1082,12 @@ cogl_atlas_texture_vtable =
     _cogl_atlas_texture_transform_quad_coords_to_gl,
     _cogl_atlas_texture_get_gl_texture,
     _cogl_atlas_texture_set_filters,
-    _cogl_atlas_texture_ensure_mipmaps,
+    _cogl_atlas_texture_pre_paint,
     _cogl_atlas_texture_ensure_non_quad_rendering,
-    _cogl_atlas_texture_set_wrap_mode_parameter,
+    _cogl_atlas_texture_set_wrap_mode_parameters,
     _cogl_atlas_texture_get_format,
     _cogl_atlas_texture_get_gl_format,
     _cogl_atlas_texture_get_width,
-    _cogl_atlas_texture_get_height
+    _cogl_atlas_texture_get_height,
+    NULL /* is_foreign */
   };
