@@ -16,9 +16,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
+ *
+ *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -33,11 +33,20 @@
 #include "cogl-texture-private.h"
 #include "cogl-material-private.h"
 #include "cogl-framebuffer-private.h"
+#include "cogl-path-private.h"
 
 #include <string.h>
 
+#ifdef HAVE_COGL_GL
+#define glActiveTexture _context->drv.pf_glActiveTexture
+#endif
+
 extern void
 _cogl_create_context_driver (CoglContext *context);
+extern void
+_cogl_create_context_winsys (CoglContext *context);
+extern void
+_cogl_destroy_context_winsys (CoglContext *context);
 
 static CoglContext *_context = NULL;
 static gboolean gl_is_indirect = FALSE;
@@ -57,17 +66,23 @@ cogl_create_context (void)
 
   /* Init default values */
   _context->feature_flags = 0;
+  _context->feature_flags_private = 0;
   _context->features_cached = FALSE;
 
+  _context->texture_types = NULL;
+
   /* Initialise the driver specific state */
   /* TODO: combine these two into one function */
   _cogl_create_context_driver (_context);
   _cogl_features_init ();
 
+  _cogl_create_context_winsys (_context);
+
   _cogl_material_init_default_material ();
+  _cogl_material_init_default_layers ();
 
   _context->enable_flags = 0;
-  _context->color_alpha = 0;
+  _context->fog_enabled = FALSE;
 
   _context->enable_backface_culling = FALSE;
   _context->flushed_front_winding = COGL_FRONT_WINDING_COUNTER_CLOCKWISE;
@@ -79,10 +94,20 @@ cogl_create_context (void)
   cogl_matrix_scale (&_context->y_flip_matrix, 1, -1, 1);
 
   _context->flushed_matrix_mode = COGL_MATRIX_MODELVIEW;
-  _context->texture_units = NULL;
+
+  _context->texture_units =
+    g_array_new (FALSE, FALSE, sizeof (CoglTextureUnit));
+
+  /* See cogl-material.c for more details about why we leave texture unit 1
+   * active by default... */
+  _context->active_texture_unit = 1;
+  GE (glActiveTexture (GL_TEXTURE1));
 
   _context->simple_material = cogl_material_new ();
   _context->source_material = NULL;
+  _context->arbfp_source_buffer = g_string_new ("");
+
+  _context->legacy_state_set = 0;
 
   _context->default_gl_texture_2d_tex = COGL_INVALID_HANDLE;
   _context->default_gl_texture_rect_tex = COGL_INVALID_HANDLE;
@@ -91,14 +116,39 @@ cogl_create_context (void)
   _context->logged_vertices = g_array_new (FALSE, FALSE, sizeof (GLfloat));
 
   _context->current_material = NULL;
-  _context->current_material_flags = 0;
-  memset (&_context->current_material_flush_options,
-          0, sizeof (CoglMaterialFlushOptions));
-  _context->current_layers = g_array_new (FALSE, FALSE,
-                                          sizeof (CoglLayerInfo));
-  _context->n_texcoord_arrays_enabled = 0;
+  _context->current_material_changes_since_flush = 0;
+  _context->current_material_skip_gl_color = FALSE;
+
+  _context->material0_nodes =
+    g_array_sized_new (FALSE, FALSE, sizeof (CoglHandle), 20);
+  _context->material1_nodes =
+    g_array_sized_new (FALSE, FALSE, sizeof (CoglHandle), 20);
+
+  _cogl_bitmask_init (&_context->texcoord_arrays_enabled);
+  _cogl_bitmask_init (&_context->temp_bitmask);
+  _cogl_bitmask_init (&_context->texcoord_arrays_to_disable);
+
+  _context->max_texture_units = -1;
+  _context->max_texture_image_units = -1;
+  _context->max_activateable_texture_units = -1;
+
+  _context->current_program = COGL_INVALID_HANDLE;
+
+  _context->current_use_program_type = COGL_MATERIAL_PROGRAM_TYPE_FIXED;
+  _context->current_gl_program = 0;
+
+  _context->gl_blend_enable_cache = FALSE;
+
+  _context->depth_test_enabled_cache = FALSE;
+  _context->depth_test_function_cache = COGL_DEPTH_TEST_FUNCTION_LESS;
+  _context->depth_writing_enabled_cache = TRUE;
+  _context->depth_range_near_cache = 0;
+  _context->depth_range_far_cache = 1;
+
+  _context->legacy_depth_test_enabled = FALSE;
 
   _context->framebuffer_stack = _cogl_create_framebuffer_stack ();
+
   window_buffer = _cogl_onscreen_new ();
   cogl_set_framebuffer (window_buffer);
   /* XXX: the deprecated _cogl_set_draw_buffer API expects to
@@ -108,8 +158,7 @@ cogl_create_context (void)
   _context->dirty_bound_framebuffer = TRUE;
   _context->dirty_gl_viewport = TRUE;
 
-  _context->path_nodes = g_array_new (FALSE, FALSE, sizeof (CoglPathNode));
-  _context->last_path = 0;
+  _context->current_path = _cogl_path_new ();
   _context->stencil_material = cogl_material_new ();
 
   _context->in_begin_gl_block = FALSE;
@@ -120,6 +169,12 @@ cogl_create_context (void)
 
   _context->texture_download_material = COGL_INVALID_HANDLE;
 
+  /* The default for GL_ALPHA_TEST is to always pass which is equivalent to
+   * the test being disabled therefore we assume that for all drivers there
+   * will be no performance impact if we always leave the test enabled which
+   * makes things a bit simpler for us. */
+  GE (glEnable (GL_ALPHA_TEST));
+
   /* Create default textures used for fall backs */
   _context->default_gl_texture_2d_tex =
     cogl_texture_new_from_data (1, /* width */
@@ -141,10 +196,8 @@ cogl_create_context (void)
                                 default_texture_data);
 
   cogl_set_source (_context->simple_material);
-  _cogl_material_flush_gl_state (_context->source_material, NULL);
-  enable_flags =
-    _cogl_material_get_cogl_enable_flags (_context->source_material);
-  cogl_enable (enable_flags);
+  _cogl_material_flush_gl_state (_context->source_material, FALSE);
+  _cogl_enable (enable_flags);
   _cogl_flush_face_winding ();
 
   _context->atlas = NULL;
@@ -152,24 +205,24 @@ cogl_create_context (void)
 
   _context->current_pbo = NULL;
 
-  _context->max_texture_units = -1;
-
   return TRUE;
 }
 
 void
-_cogl_destroy_context ()
+_cogl_destroy_context (void)
 {
 
   if (_context == NULL)
     return;
 
+  _cogl_destroy_context_winsys (_context);
+
   _cogl_destroy_texture_units ();
 
   _cogl_free_framebuffer_stack (_context->framebuffer_stack);
 
-  if (_context->path_nodes)
-    g_array_free (_context->path_nodes, TRUE);
+  if (_context->current_path)
+    cogl_handle_unref (_context->current_path);
 
   if (_context->default_gl_texture_2d_tex)
     cogl_handle_unref (_context->default_gl_texture_2d_tex);
@@ -184,9 +237,6 @@ _cogl_destroy_context ()
   if (_context->logged_vertices)
     g_array_free (_context->logged_vertices, TRUE);
 
-  if (_context->current_layers)
-    g_array_free (_context->current_layers, TRUE);
-
   if (_context->quad_indices_byte)
     cogl_handle_unref (_context->quad_indices_byte);
   if (_context->quad_indices_short)
@@ -195,16 +245,29 @@ _cogl_destroy_context ()
   if (_context->default_material)
     cogl_handle_unref (_context->default_material);
 
+  if (_context->dummy_layer_dependant)
+    cogl_handle_unref (_context->dummy_layer_dependant);
+  if (_context->default_layer_n)
+    cogl_handle_unref (_context->default_layer_n);
+  if (_context->default_layer_0)
+    cogl_handle_unref (_context->default_layer_0);
+
   if (_context->atlas)
     _cogl_atlas_free (_context->atlas);
   if (_context->atlas_texture)
     cogl_handle_unref (_context->atlas_texture);
 
+  _cogl_bitmask_destroy (&_context->texcoord_arrays_enabled);
+  _cogl_bitmask_destroy (&_context->temp_bitmask);
+  _cogl_bitmask_destroy (&_context->texcoord_arrays_to_disable);
+
+  g_slist_free (_context->texture_types);
+
   g_free (_context);
 }
 
 CoglContext *
-_cogl_context_get_default ()
+_cogl_context_get_default (void)
 {
   /* Create if doesn't exist yet */
   if (_context == NULL)
