@@ -42,7 +42,7 @@
 
 static void _cogl_sub_texture_free (CoglSubTexture *sub_tex);
 
-COGL_HANDLE_DEFINE (SubTexture, sub_texture);
+COGL_TEXTURE_DEFINE (SubTexture, sub_texture);
 
 static const CoglTextureVtable cogl_sub_texture_vtable;
 
@@ -243,7 +243,8 @@ _cogl_sub_texture_free (CoglSubTexture *sub_tex)
   cogl_handle_unref (sub_tex->next_texture);
   cogl_handle_unref (sub_tex->full_texture);
 
-  g_free (sub_tex);
+  /* Chain up */
+  _cogl_texture_free (COGL_TEXTURE (sub_tex));
 }
 
 CoglHandle
@@ -384,11 +385,12 @@ _cogl_sub_texture_set_filters (CoglTexture *tex,
 }
 
 static void
-_cogl_sub_texture_ensure_mipmaps (CoglTexture *tex)
+_cogl_sub_texture_pre_paint (CoglTexture *tex,
+                             CoglTexturePrePaintFlags flags)
 {
   CoglSubTexture *sub_tex = COGL_SUB_TEXTURE (tex);
 
-  _cogl_texture_ensure_mipmaps (sub_tex->full_texture);
+  _cogl_texture_pre_paint (sub_tex->full_texture, flags);
 }
 
 static void
@@ -550,11 +552,12 @@ cogl_sub_texture_vtable =
     _cogl_sub_texture_transform_quad_coords_to_gl,
     _cogl_sub_texture_get_gl_texture,
     _cogl_sub_texture_set_filters,
-    _cogl_sub_texture_ensure_mipmaps,
+    _cogl_sub_texture_pre_paint,
     _cogl_sub_texture_ensure_non_quad_rendering,
     _cogl_sub_texture_set_wrap_mode_parameters,
     _cogl_sub_texture_get_format,
     _cogl_sub_texture_get_gl_format,
     _cogl_sub_texture_get_width,
-    _cogl_sub_texture_get_height
+    _cogl_sub_texture_get_height,
+    NULL /* is_foreign */
   };
