@@ -16,9 +16,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
+ *
+ *
  */
 
 #ifndef __COGL_TEXTURE_2D_SLICED_H
@@ -65,7 +65,8 @@ struct _CoglTexture2DSliced
   GLenum            min_filter;
   GLenum            mag_filter;
   gboolean          is_foreign;
-  GLint             wrap_mode;
+  GLenum            wrap_mode_s;
+  GLenum            wrap_mode_t;
   gboolean          auto_mipmap;
   gboolean          mipmaps_dirty;
 
