@@ -42,6 +42,7 @@
 #include "cogl-handle.h"
 #include "cogl-spans.h"
 #include "cogl-journal-private.h"
+#include "cogl-material-private.h"
 
 #include <string.h>
 #include <stdlib.h>
@@ -49,7 +50,7 @@
 
 static void _cogl_texture_2d_sliced_free (CoglTexture2DSliced *tex_2ds);
 
-COGL_HANDLE_DEFINE (Texture2DSliced, texture_2d_sliced);
+COGL_TEXTURE_DEFINE (Texture2DSliced, texture_2d_sliced);
 
 static const CoglTextureVtable cogl_texture_2d_sliced_vtable;
 
@@ -253,6 +254,7 @@ _cogl_texture_2d_sliced_upload_to_gl (CoglTexture2DSliced *tex_2ds,
           _cogl_texture_driver_upload_subregion_to_gl (
                                      tex_2ds->gl_target,
                                      gl_handle,
+                                     tex_2ds->is_foreign,
                                      x_span->start, /* src x */
                                      y_span->start, /* src y */
                                      0, /* dst x */
@@ -441,6 +443,7 @@ _cogl_texture_2d_sliced_upload_subregion_to_gl (CoglTexture2DSliced *tex_2ds,
 
           _cogl_texture_driver_upload_subregion_to_gl (tex_2ds->gl_target,
                                                        gl_handle,
+                                                       tex_2ds->is_foreign,
                                                        source_x,
                                                        source_y,
                                                        local_x, /* dst x */
@@ -661,31 +664,36 @@ _cogl_pot_slices_for_size (int    size_to_fill,
 }
 
 static void
-_cogl_texture_2d_sliced_set_wrap_mode_parameter (CoglTexture *tex,
-                                                 GLenum wrap_mode)
+_cogl_texture_2d_sliced_set_wrap_mode_parameters (CoglTexture *tex,
+                                                  GLenum wrap_mode_s,
+                                                  GLenum wrap_mode_t,
+                                                  GLenum wrap_mode_r)
 {
   CoglTexture2DSliced *tex_2ds = COGL_TEXTURE_2D_SLICED (tex);
 
-  /* Only set the wrap mode if it's different from the current
-     value to avoid too many GL calls */
-  if (tex_2ds->wrap_mode != wrap_mode)
+  /* Only set the wrap mode if it's different from the current value
+     to avoid too many GL calls. Texture 2D doesn't make use of the r
+     coordinate so we can ignore its wrap mode */
+  if (tex_2ds->wrap_mode_s != wrap_mode_s ||
+      tex_2ds->wrap_mode_t != wrap_mode_t)
     {
       int i;
 
-      /* Any queued texture rectangles may be depending on the previous
-       * wrap mode... */
-      _cogl_journal_flush ();
-
       for (i = 0; i < tex_2ds->slice_gl_handles->len; i++)
         {
           GLuint texnum = g_array_index (tex_2ds->slice_gl_handles, GLuint, i);
 
-          GE( glBindTexture (tex_2ds->gl_target, texnum) );
-          GE( glTexParameteri (tex_2ds->gl_target, GL_TEXTURE_WRAP_S, wrap_mode) );
-          GE( glTexParameteri (tex_2ds->gl_target, GL_TEXTURE_WRAP_T, wrap_mode) );
+          _cogl_bind_gl_texture_transient (tex_2ds->gl_target,
+                                           texnum,
+                                           tex_2ds->is_foreign);
+          GE( glTexParameteri (tex_2ds->gl_target,
+                               GL_TEXTURE_WRAP_S, wrap_mode_s) );
+          GE( glTexParameteri (tex_2ds->gl_target,
+                               GL_TEXTURE_WRAP_T, wrap_mode_t) );
         }
 
-      tex_2ds->wrap_mode = wrap_mode;
+      tex_2ds->wrap_mode_s = wrap_mode_s;
+      tex_2ds->wrap_mode_t = wrap_mode_t;
     }
 }
 
@@ -827,7 +835,8 @@ _cogl_texture_2d_sliced_slices_create (CoglTexture2DSliced *tex_2ds,
     tex_2ds->first_pixels = g_new (CoglTexturePixel, n_slices);
 
   /* Wrap mode not yet set */
-  tex_2ds->wrap_mode = GL_FALSE;
+  tex_2ds->wrap_mode_s = GL_FALSE;
+  tex_2ds->wrap_mode_t = GL_FALSE;
 
   /* Generate a "working set" of GL texture objects
    * (some implementations might supported faster
@@ -851,8 +860,9 @@ _cogl_texture_2d_sliced_slices_create (CoglTexture2DSliced *tex_2ds,
                      y_span->size - y_span->waste);
 
           /* Setup texture parameters */
-          GE( glBindTexture (tex_2ds->gl_target,
-                             gl_handles[y * n_x_slices + x] ) );
+          _cogl_bind_gl_texture_transient (tex_2ds->gl_target,
+                                           gl_handles[y * n_x_slices + x],
+                                           FALSE);
 
           _cogl_texture_driver_try_setting_gl_border_color (tex_2ds->gl_target,
                                                             transparent_color);
@@ -860,7 +870,7 @@ _cogl_texture_2d_sliced_slices_create (CoglTexture2DSliced *tex_2ds,
           /* Pass NULL data to init size and internal format */
           GE( glTexImage2D (tex_2ds->gl_target, 0, gl_intformat,
                             x_span->size, y_span->size, 0,
-                            gl_format, gl_type, 0) );
+                            gl_format, gl_type, NULL) );
         }
     }
 
@@ -878,11 +888,14 @@ _cogl_texture_2d_sliced_slices_free (CoglTexture2DSliced *tex_2ds)
 
   if (tex_2ds->slice_gl_handles != NULL)
     {
+      int i;
       if (tex_2ds->is_foreign == FALSE)
-        {
-          GE( glDeleteTextures (tex_2ds->slice_gl_handles->len,
-                                (GLuint*) tex_2ds->slice_gl_handles->data) );
-        }
+        for (i = 0; i < tex_2ds->slice_gl_handles->len; i++)
+          {
+            GLuint texture =
+              g_array_index (tex_2ds->slice_gl_handles, GLuint, i);
+            _cogl_delete_gl_texture (texture);
+          }
 
       g_array_free (tex_2ds->slice_gl_handles, TRUE);
     }
@@ -895,7 +908,9 @@ static void
 _cogl_texture_2d_sliced_free (CoglTexture2DSliced *tex_2ds)
 {
   _cogl_texture_2d_sliced_slices_free (tex_2ds);
-  g_free (tex_2ds);
+
+  /* Chain up */
+  _cogl_texture_free (COGL_TEXTURE (tex_2ds));
 }
 
 static gboolean
@@ -941,8 +956,8 @@ _cogl_texture_2d_sliced_upload_from_data
 
       /* Create slices for the given format and size */
       if (!_cogl_texture_2d_sliced_slices_create (tex_2ds,
-                                                  bmp->width,
-                                                  bmp->height,
+                                                  dst_bmp.width,
+                                                  dst_bmp.height,
                                                   gl_intformat,
                                                   gl_format,
                                                   gl_type))
@@ -954,7 +969,7 @@ _cogl_texture_2d_sliced_upload_from_data
         }
 
       if (!_cogl_texture_2d_sliced_upload_to_gl (tex_2ds,
-                                                 bmp,
+                                                 &dst_bmp,
                                                  gl_intformat,
                                                  gl_format,
                                                  gl_type))
@@ -1117,7 +1132,8 @@ _cogl_texture_2d_sliced_new_from_foreign (GLuint           gl_handle,
   /* Make sure binding succeeds */
   while ((gl_error = glGetError ()) != GL_NO_ERROR)
     ;
-  glBindTexture (gl_target, gl_handle);
+
+  _cogl_bind_gl_texture_transient (gl_target, gl_handle, TRUE);
   if (glGetError () != GL_NO_ERROR)
     return COGL_INVALID_HANDLE;
 
@@ -1188,7 +1204,8 @@ _cogl_texture_2d_sliced_new_from_foreign (GLuint           gl_handle,
   tex_2ds->max_waste = 0;
 
   /* Wrap mode not yet set */
-  tex_2ds->wrap_mode = GL_FALSE;
+  tex_2ds->wrap_mode_s = GL_FALSE;
+  tex_2ds->wrap_mode_t = GL_FALSE;
 
   /* Create slice arrays */
   tex_2ds->slice_x_spans =
@@ -1221,6 +1238,12 @@ _cogl_texture_2d_sliced_new_from_foreign (GLuint           gl_handle,
   return _cogl_texture_2d_sliced_handle_new (tex_2ds);
 }
 
+static gboolean
+_cogl_texture_2d_sliced_is_foreign (CoglTexture *tex)
+{
+  return COGL_TEXTURE_2D_SLICED (tex)->is_foreign;
+}
+
 static int
 _cogl_texture_2d_sliced_get_max_waste (CoglTexture *tex)
 {
@@ -1366,7 +1389,9 @@ _cogl_texture_2d_sliced_set_filters (CoglTexture *tex,
   for (i=0; i<tex_2ds->slice_gl_handles->len; ++i)
     {
       gl_handle = g_array_index (tex_2ds->slice_gl_handles, GLuint, i);
-      GE( glBindTexture (tex_2ds->gl_target, gl_handle) );
+      _cogl_bind_gl_texture_transient (tex_2ds->gl_target,
+                                       gl_handle,
+                                       tex_2ds->is_foreign);
       GE( glTexParameteri (tex_2ds->gl_target, GL_TEXTURE_MAG_FILTER,
                            tex_2ds->mag_filter) );
       GE( glTexParameteri (tex_2ds->gl_target, GL_TEXTURE_MIN_FILTER,
@@ -1375,13 +1400,18 @@ _cogl_texture_2d_sliced_set_filters (CoglTexture *tex,
 }
 
 static void
-_cogl_texture_2d_sliced_ensure_mipmaps (CoglTexture *tex)
+_cogl_texture_2d_sliced_pre_paint (CoglTexture *tex,
+                                   CoglTexturePrePaintFlags flags)
 {
   CoglTexture2DSliced *tex_2ds = COGL_TEXTURE_2D_SLICED (tex);
   int                  i;
 
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
+  /* We only care about the mipmap flag */
+  if ((flags & COGL_TEXTURE_NEEDS_MIPMAP) == 0)
+    return;
+
   /* Only update if the mipmaps are dirty */
   if (!tex_2ds->auto_mipmap || !tex_2ds->mipmaps_dirty)
     return;
@@ -1394,7 +1424,9 @@ _cogl_texture_2d_sliced_ensure_mipmaps (CoglTexture *tex)
   for (i = 0; i < tex_2ds->slice_gl_handles->len; i++)
     {
       GLuint gl_handle = g_array_index (tex_2ds->slice_gl_handles, GLuint, i);
-      GE( glBindTexture (tex_2ds->gl_target, gl_handle) );
+      _cogl_bind_gl_texture_transient (tex_2ds->gl_target,
+                                       gl_handle,
+                                       tex_2ds->is_foreign);
 
       /* glGenerateMipmap is defined in the FBO extension */
       if (cogl_features_available (COGL_FEATURE_OFFSCREEN))
@@ -1539,7 +1571,9 @@ _cogl_texture_2d_sliced_download_from_gl (
                                                           bpp);
 
 	      /* Download slice image data into temp bmp */
-	      GE( glBindTexture (tex_2ds->gl_target, gl_handle) );
+              _cogl_bind_gl_texture_transient (tex_2ds->gl_target,
+                                               gl_handle,
+                                               tex_2ds->is_foreign);
 
               if (!_cogl_texture_driver_gl_get_tex_image (tex_2ds->gl_target,
                                                           target_gl_format,
@@ -1573,7 +1607,9 @@ _cogl_texture_2d_sliced_download_from_gl (
                                                         bpp);
 
 	      /* Download slice image data */
-	      GE( glBindTexture (tex_2ds->gl_target, gl_handle) );
+              _cogl_bind_gl_texture_transient (tex_2ds->gl_target,
+                                               gl_handle,
+                                               tex_2ds->is_foreign);
 
               if (!_cogl_texture_driver_gl_get_tex_image (tex_2ds->gl_target,
                                                           target_gl_format,
@@ -1679,7 +1715,7 @@ _cogl_texture_2d_sliced_get_data (CoglTexture     *tex,
         {
           src = new_bmp.data + y * new_bmp.rowstride;
           dst = data + y * rowstride;
-          memcpy (dst, src, new_bmp.width);
+          memcpy (dst, src, new_bmp.rowstride);
         }
 
       /* Free converted data */
@@ -1726,11 +1762,12 @@ cogl_texture_2d_sliced_vtable =
     _cogl_texture_2d_sliced_transform_quad_coords_to_gl,
     _cogl_texture_2d_sliced_get_gl_texture,
     _cogl_texture_2d_sliced_set_filters,
-    _cogl_texture_2d_sliced_ensure_mipmaps,
+    _cogl_texture_2d_sliced_pre_paint,
     _cogl_texture_2d_sliced_ensure_non_quad_rendering,
-    _cogl_texture_2d_sliced_set_wrap_mode_parameter,
+    _cogl_texture_2d_sliced_set_wrap_mode_parameters,
     _cogl_texture_2d_sliced_get_format,
     _cogl_texture_2d_sliced_get_gl_format,
     _cogl_texture_2d_sliced_get_width,
-    _cogl_texture_2d_sliced_get_height
+    _cogl_texture_2d_sliced_get_height,
+    _cogl_texture_2d_sliced_is_foreign
   };
