@@ -37,13 +37,14 @@
 #include "cogl-context.h"
 #include "cogl-handle.h"
 #include "cogl-journal-private.h"
+#include "cogl-material-private.h"
 
 #include <string.h>
 #include <math.h>
 
 static void _cogl_texture_2d_free (CoglTexture2D *tex_2d);
 
-COGL_HANDLE_DEFINE (Texture2D, texture_2d);
+COGL_TEXTURE_DEFINE (Texture2D, texture_2d);
 
 static const CoglTextureVtable cogl_texture_2d_vtable;
 
@@ -137,7 +138,9 @@ _cogl_texture_2d_set_wrap_mode_parameters (CoglTexture *tex,
   if (tex_2d->wrap_mode_s != wrap_mode_s ||
       tex_2d->wrap_mode_t != wrap_mode_t)
     {
-      GE( glBindTexture (GL_TEXTURE_2D, tex_2d->gl_texture) );
+      _cogl_bind_gl_texture_transient (GL_TEXTURE_2D,
+                                       tex_2d->gl_texture,
+                                       FALSE);
       GE( glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, wrap_mode_s) );
       GE( glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, wrap_mode_t) );
 
@@ -149,8 +152,10 @@ _cogl_texture_2d_set_wrap_mode_parameters (CoglTexture *tex,
 static void
 _cogl_texture_2d_free (CoglTexture2D *tex_2d)
 {
-  GE( glDeleteTextures (1, &tex_2d->gl_texture) );
-  g_free (tex_2d);
+  _cogl_delete_gl_texture (tex_2d->gl_texture);
+
+  /* Chain up */
+  _cogl_texture_free (COGL_TEXTURE (tex_2d));
 }
 
 static gboolean
@@ -253,7 +258,9 @@ _cogl_texture_2d_new_with_size (unsigned int     width,
   tex_2d = _cogl_texture_2d_create_base (width, height, flags, internal_format);
 
   _cogl_texture_driver_gen (GL_TEXTURE_2D, 1, &tex_2d->gl_texture);
-  GE( glBindTexture (GL_TEXTURE_2D, tex_2d->gl_texture) );
+  _cogl_bind_gl_texture_transient (GL_TEXTURE_2D,
+                                   tex_2d->gl_texture,
+                                   FALSE);
   GE( glTexImage2D (GL_TEXTURE_2D, 0, gl_intformat,
                     width, height, 0, gl_format, gl_type, NULL) );
 
@@ -299,6 +306,7 @@ _cogl_texture_2d_new_from_bitmap (CoglHandle       bmp_handle,
   _cogl_texture_driver_gen (GL_TEXTURE_2D, 1, &tex_2d->gl_texture);
   _cogl_texture_driver_upload_to_gl (GL_TEXTURE_2D,
                                      tex_2d->gl_texture,
+                                     FALSE,
                                      &dst_bmp,
                                      gl_intformat,
                                      gl_format,
@@ -312,6 +320,15 @@ _cogl_texture_2d_new_from_bitmap (CoglHandle       bmp_handle,
   return _cogl_texture_2d_handle_new (tex_2d);
 }
 
+void
+_cogl_texture_2d_externally_modified (CoglHandle handle)
+{
+  if (!cogl_is_texture_2d (handle))
+    return;
+
+  COGL_TEXTURE_2D (handle)->mipmaps_dirty = TRUE;
+}
+
 static int
 _cogl_texture_2d_get_max_waste (CoglTexture *tex)
 {
@@ -389,29 +406,34 @@ _cogl_texture_2d_set_filters (CoglTexture *tex,
   tex_2d->mag_filter = mag_filter;
 
   /* Apply new filters to the texture */
-  GE( glBindTexture (GL_TEXTURE_2D, tex_2d->gl_texture) );
+  _cogl_bind_gl_texture_transient (GL_TEXTURE_2D,
+                                   tex_2d->gl_texture,
+                                   FALSE);
   GE( glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, mag_filter) );
   GE( glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, min_filter) );
 }
 
 static void
-_cogl_texture_2d_ensure_mipmaps (CoglTexture *tex)
+_cogl_texture_2d_pre_paint (CoglTexture *tex, CoglTexturePrePaintFlags flags)
 {
   CoglTexture2D *tex_2d = COGL_TEXTURE_2D (tex);
 
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
   /* Only update if the mipmaps are dirty */
-  if (!tex_2d->auto_mipmap || !tex_2d->mipmaps_dirty)
-    return;
+  if ((flags & COGL_TEXTURE_NEEDS_MIPMAP) &&
+      tex_2d->auto_mipmap && tex_2d->mipmaps_dirty)
+    {
+      _cogl_bind_gl_texture_transient (GL_TEXTURE_2D,
+                                       tex_2d->gl_texture,
+                                       FALSE);
+      /* glGenerateMipmap is defined in the FBO extension. We only allow
+         CoglTexture2D instances to be created if this feature is
+         available so we don't need to check for the extension */
+      _cogl_texture_driver_gl_generate_mipmaps (GL_TEXTURE_2D);
 
-  GE( glBindTexture (GL_TEXTURE_2D, tex_2d->gl_texture) );
-  /* glGenerateMipmap is defined in the FBO extension. We only allow
-     CoglTexture2D instances to be created if this feature is
-     available so we don't need to check for the extension */
-  _cogl_texture_driver_gl_generate_mipmaps (GL_TEXTURE_2D);
-
-  tex_2d->mipmaps_dirty = FALSE;
+      tex_2d->mipmaps_dirty = FALSE;
+    }
 }
 
 static void
@@ -474,6 +496,7 @@ _cogl_texture_2d_set_region (CoglTexture    *tex,
   /* Send data to GL */
   _cogl_texture_driver_upload_subregion_to_gl (GL_TEXTURE_2D,
                                                tex_2d->gl_texture,
+                                               FALSE,
                                                src_x, src_y,
                                                dst_x, dst_y,
                                                dst_width, dst_height,
@@ -485,6 +508,8 @@ _cogl_texture_2d_set_region (CoglTexture    *tex,
   if (tmp_bmp_owner)
     g_free (tmp_bmp.data);
 
+  tex_2d->mipmaps_dirty = TRUE;
+
   return TRUE;
 }
 
@@ -550,7 +575,9 @@ _cogl_texture_2d_get_data (CoglTexture     *tex,
   _cogl_texture_driver_prep_gl_for_pixels_download (target_bmp.rowstride,
                                                     closest_bpp);
 
-  GE( glBindTexture (GL_TEXTURE_2D, tex_2d->gl_texture) );
+  _cogl_bind_gl_texture_transient (GL_TEXTURE_2D,
+                                   tex_2d->gl_texture,
+                                   FALSE);
   if (!_cogl_texture_driver_gl_get_tex_image (GL_TEXTURE_2D,
                                               closest_gl_format,
                                               closest_gl_type,
@@ -630,11 +657,12 @@ cogl_texture_2d_vtable =
     _cogl_texture_2d_transform_quad_coords_to_gl,
     _cogl_texture_2d_get_gl_texture,
     _cogl_texture_2d_set_filters,
-    _cogl_texture_2d_ensure_mipmaps,
+    _cogl_texture_2d_pre_paint,
     _cogl_texture_2d_ensure_non_quad_rendering,
     _cogl_texture_2d_set_wrap_mode_parameters,
     _cogl_texture_2d_get_format,
     _cogl_texture_2d_get_gl_format,
     _cogl_texture_2d_get_width,
-    _cogl_texture_2d_get_height
+    _cogl_texture_2d_get_height,
+    NULL /* is_foreign */
   };
