@@ -59,18 +59,31 @@
  * abstract class manually.
  */
 
+void
+_cogl_texture_register_texture_type (GQuark type)
+{
+  _COGL_GET_CONTEXT (ctxt, NO_RETVAL);
+
+  ctxt->texture_types = g_slist_prepend (ctxt->texture_types,
+                                         GINT_TO_POINTER (type));
+}
+
 gboolean
 cogl_is_texture (CoglHandle handle)
 {
   CoglHandleObject *obj = (CoglHandleObject *)handle;
+  GSList *l;
+
+  _COGL_GET_CONTEXT (ctxt, FALSE);
 
   if (handle == COGL_INVALID_HANDLE)
     return FALSE;
 
-  return (obj->klass->type == _cogl_handle_texture_2d_get_type () ||
-          obj->klass->type == _cogl_handle_atlas_texture_get_type () ||
-          obj->klass->type == _cogl_handle_texture_2d_sliced_get_type () ||
-          obj->klass->type == _cogl_handle_sub_texture_get_type ());
+  for (l = ctxt->texture_types; l; l = l->next)
+    if (GPOINTER_TO_INT (l->data) == obj->klass->type)
+      return TRUE;
+
+  return FALSE;
 }
 
 CoglHandle
@@ -102,12 +115,19 @@ cogl_texture_unref (CoglHandle handle)
   cogl_handle_unref (handle);
 }
 
+void
+_cogl_texture_free (CoglTexture *texture)
+{
+  g_free (texture);
+}
+
 static gboolean
 _cogl_texture_needs_premult_conversion (CoglPixelFormat src_format,
                                         CoglPixelFormat dst_format)
 {
-  return ((src_format & COGL_A_BIT) &&
+  return ((src_format & dst_format & COGL_A_BIT) &&
           src_format != COGL_PIXEL_FORMAT_A_8 &&
+          dst_format != COGL_PIXEL_FORMAT_A_8 &&
           (src_format & COGL_PREMULT_BIT) !=
           (dst_format & COGL_PREMULT_BIT));
 }
@@ -471,6 +491,8 @@ cogl_texture_new_from_foreign (GLuint           gl_handle,
 			       GLuint           y_pot_waste,
 			       CoglPixelFormat  format)
 {
+  /* FIXME: only create a sliced texture if x or y waste was specified
+   */
   return _cogl_texture_2d_sliced_new_from_foreign (gl_handle,
                                                    gl_target,
                                                    width,
@@ -480,6 +502,21 @@ cogl_texture_new_from_foreign (GLuint           gl_handle,
                                                    format);
 }
 
+gboolean
+_cogl_texture_is_foreign (CoglHandle handle)
+{
+  CoglTexture *tex;
+
+  g_return_val_if_fail (cogl_is_texture (handle), FALSE);
+
+  tex = COGL_TEXTURE (handle);
+
+  if (tex->vtable->is_foreign)
+    return tex->vtable->is_foreign (tex);
+  else
+    return FALSE;
+}
+
 CoglHandle
 cogl_texture_new_from_sub_texture (CoglHandle full_texture,
                                    int        sub_x,
@@ -749,7 +786,7 @@ _cogl_texture_set_filters (CoglHandle handle,
 }
 
 void
-_cogl_texture_ensure_mipmaps (CoglHandle handle)
+_cogl_texture_pre_paint (CoglHandle handle, CoglTexturePrePaintFlags flags)
 {
   CoglTexture *tex;
 
@@ -758,7 +795,7 @@ _cogl_texture_ensure_mipmaps (CoglHandle handle)
 
   tex = COGL_TEXTURE (handle);
 
-  tex->vtable->ensure_mipmaps (tex);
+  tex->vtable->pre_paint (tex, flags);
 }
 
 void
