@@ -159,7 +159,6 @@
 #define glDeleteBuffers ctx->drv.pf_glDeleteBuffers
 #define glMapBuffer ctx->drv.pf_glMapBuffer
 #define glUnmapBuffer ctx->drv.pf_glUnmapBuffer
-#define glActiveTexture ctx->drv.pf_glActiveTexture
 #define glClientActiveTexture ctx->drv.pf_glClientActiveTexture
 #ifndef GL_ARRAY_BUFFER
 #define GL_ARRAY_BUFFER GL_ARRAY_BUFFER_ARB
@@ -1509,7 +1508,7 @@ get_gl_type_from_attribute_flags (CoglVertexBufferAttribFlags flags)
     }
 }
 
-static void
+static CoglHandle
 enable_state_for_drawing_buffer (CoglVertexBuffer *buffer)
 {
   GList       *tmp;
@@ -1521,9 +1520,13 @@ enable_state_for_drawing_buffer (CoglVertexBuffer *buffer)
   const GList *layers;
   guint32      fallback_layers = 0;
   int          i;
+  gboolean     skip_gl_color = FALSE;
   CoglMaterialFlushOptions options;
+  CoglHandle   source;
 
-  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
+  _COGL_GET_CONTEXT (ctx, COGL_INVALID_HANDLE);
+
+  source = ctx->source_material;
 
   if (buffer->new_attributes)
     cogl_vertex_buffer_submit_real (buffer);
@@ -1572,13 +1575,22 @@ enable_state_for_drawing_buffer (CoglVertexBuffer *buffer)
 	  switch (type)
 	    {
 	    case COGL_VERTEX_BUFFER_ATTRIB_FLAG_COLOR_ARRAY:
-	      enable_flags |= COGL_ENABLE_COLOR_ARRAY | COGL_ENABLE_BLEND;
+	      enable_flags |= COGL_ENABLE_COLOR_ARRAY;
 	      /* GE (glEnableClientState (GL_COLOR_ARRAY)); */
 	      pointer = (const GLvoid *)(base + attribute->u.vbo_offset);
 	      GE (glColorPointer (attribute->n_components,
 				  gl_type,
 				  attribute->stride,
 				  pointer));
+
+              if (!_cogl_material_get_real_blend_enabled (ctx->source_material))
+                {
+                  CoglMaterialBlendEnable blend_enable =
+                    COGL_MATERIAL_BLEND_ENABLE_ENABLED;
+                  source = cogl_material_copy (ctx->source_material);
+                  _cogl_material_set_blend_enabled (source, blend_enable);
+                  skip_gl_color = TRUE;
+                }
 	      break;
 	    case COGL_VERTEX_BUFFER_ATTRIB_FLAG_NORMAL_ARRAY:
 	      /* FIXME: go through cogl cache to enable normal array */
@@ -1634,7 +1646,7 @@ enable_state_for_drawing_buffer (CoglVertexBuffer *buffer)
 	}
     }
 
-  layers = cogl_material_get_layers (ctx->source_material);
+  layers = cogl_material_get_layers (source);
   for (tmp = (GList *)layers, i = 0;
        tmp != NULL;
        tmp = tmp->next, i++)
@@ -1652,6 +1664,13 @@ enable_state_for_drawing_buffer (CoglVertexBuffer *buffer)
          will be migrated */
       _cogl_texture_ensure_non_quad_rendering (tex_handle);
 
+      /* We need to ensure the mipmaps are ready before deciding
+       * anything else about the texture because the texture storate
+       * could completely change if it needs to be migrated out of the
+       * atlas and will affect how we validate the layer.
+       */
+      _cogl_material_layer_pre_paint (layer);
+
       if (!_cogl_texture_can_hardware_repeat (tex_handle))
         {
           g_warning ("Disabling layer %d of the current source material, "
@@ -1710,18 +1729,69 @@ enable_state_for_drawing_buffer (CoglVertexBuffer *buffer)
 
   options.fallback_layers = fallback_layers;
 
-  _cogl_material_flush_gl_state (ctx->source_material, &options);
-  enable_flags |= _cogl_material_get_cogl_enable_flags (ctx->source_material);
+  if (G_UNLIKELY (options.flags))
+    {
+      /* If we haven't already created a derived material... */
+      if (source == ctx->source_material)
+        source = cogl_material_copy (ctx->source_material);
+      _cogl_material_apply_overrides (source, &options);
+
+      /* TODO:
+       * overrides = cogl_material_get_data (material,
+       *                                     last_overrides_key);
+       * if (overrides)
+       *   {
+       *     age = cogl_material_get_age (material);
+       *     XXX: actually we also need to check for legacy_state
+       *     and blending overrides for use of glColorPointer...
+       *     if (overrides->ags != age ||
+       *         memcmp (&overrides->options, &options,
+       *                 sizeof (options) != 0)
+       *       {
+       *         cogl_handle_unref (overrides->weak_material);
+       *         g_slice_free (Overrides, overrides);
+       *         overrides = NULL;
+       *       }
+       *   }
+       * if (!overrides)
+       *   {
+       *     overrides = g_slice_new (Overrides);
+       *     overrides->weak_material =
+       *       cogl_material_weak_copy (ctx->source_material);
+       *     _cogl_material_apply_overrides (overrides->weak_material,
+       *                                     &options);
+       *
+       *     cogl_material_set_data (material, last_overrides_key,
+       *                             weak_overrides,
+       *                             free_overrides_cb,
+       *                             NULL);
+       *   }
+       * source = overrides->weak_material;
+       */
+    }
+
+  if (G_UNLIKELY (ctx->legacy_state_set))
+    {
+      /* If we haven't already created a derived material... */
+      if (source == ctx->source_material)
+        source = cogl_material_copy (ctx->source_material);
+      _cogl_material_apply_legacy_state (source);
+    }
+
+  _cogl_material_flush_gl_state (source, skip_gl_color);
 
   if (ctx->enable_backface_culling)
     enable_flags |= COGL_ENABLE_BACKFACE_CULLING;
 
   _cogl_enable (enable_flags);
   _cogl_flush_face_winding ();
+
+  return source;
 }
 
 static void
-disable_state_for_drawing_buffer (CoglVertexBuffer *buffer)
+disable_state_for_drawing_buffer (CoglVertexBuffer *buffer,
+                                  CoglHandle source)
 {
   GList *tmp;
   GLenum gl_type;
@@ -1731,6 +1801,9 @@ disable_state_for_drawing_buffer (CoglVertexBuffer *buffer)
 
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
+  if (G_UNLIKELY (source != ctx->source_material))
+    cogl_handle_unref (source);
+
   /* Disable all the client state that cogl doesn't currently know
    * about:
    */
@@ -1791,6 +1864,7 @@ cogl_vertex_buffer_draw (CoglHandle       handle,
 		         int              count)
 {
   CoglVertexBuffer *buffer;
+  CoglHandle source;
 
   if (!cogl_is_vertex_buffer (handle))
     return;
@@ -1799,11 +1873,11 @@ cogl_vertex_buffer_draw (CoglHandle       handle,
 
   buffer = _cogl_vertex_buffer_pointer_from_handle (handle);
 
-  enable_state_for_drawing_buffer (buffer);
+  source = enable_state_for_drawing_buffer (buffer);
 
   GE (glDrawArrays (mode, first, count));
 
-  disable_state_for_drawing_buffer (buffer);
+  disable_state_for_drawing_buffer (buffer, source);
 }
 
 static int
@@ -1927,6 +2001,7 @@ cogl_vertex_buffer_draw_elements (CoglHandle       handle,
     (cogl_get_features () & COGL_FEATURE_VBOS) ? FALSE : TRUE;
   gsize byte_offset;
   CoglVertexBufferIndices *indices = NULL;
+  CoglHandle source;
 
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
@@ -1942,7 +2017,7 @@ cogl_vertex_buffer_draw_elements (CoglHandle       handle,
 
   indices = _cogl_vertex_buffer_indices_pointer_from_handle (indices_handle);
 
-  enable_state_for_drawing_buffer (buffer);
+  source = enable_state_for_drawing_buffer (buffer);
 
   byte_offset = indices_offset * get_indices_type_size (indices->type);
   if (fallback)
@@ -1954,7 +2029,7 @@ cogl_vertex_buffer_draw_elements (CoglHandle       handle,
   GE (glDrawRangeElements (mode, min_index, max_index,
                            count, indices->type, (void *)byte_offset));
 
-  disable_state_for_drawing_buffer (buffer);
+  disable_state_for_drawing_buffer (buffer, source);
 
   GE (glBindBuffer (GL_ELEMENT_ARRAY_BUFFER, 0));
 }
