@@ -266,10 +266,6 @@ _cogl_enable (unsigned long flags)
   */
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
-  toggle_flag (ctx, flags,
-               COGL_ENABLE_BLEND,
-               GL_BLEND);
-
   toggle_flag (ctx, flags,
                COGL_ENABLE_BACKFACE_CULLING,
                GL_CULL_FACE);
@@ -291,25 +287,28 @@ _cogl_get_enable (void)
   return ctx->enable_flags;
 }
 
+/* XXX: This API has been deprecated */
 void
 cogl_set_depth_test_enabled (gboolean setting)
 {
-  /* Currently the journal can't track changes to depth state... */
-  _cogl_journal_flush ();
+  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
-  if (setting)
-    {
-      glEnable (GL_DEPTH_TEST);
-      glDepthFunc (GL_LEQUAL);
-    }
+  if (ctx->legacy_depth_test_enabled == setting)
+    return;
+
+  ctx->legacy_depth_test_enabled = setting;
+  if (ctx->legacy_depth_test_enabled)
+    ctx->legacy_state_set++;
   else
-    glDisable (GL_DEPTH_TEST);
+    ctx->legacy_state_set--;
 }
 
+/* XXX: This API has been deprecated */
 gboolean
 cogl_get_depth_test_enabled (void)
 {
-  return glIsEnabled (GL_DEPTH_TEST) == GL_TRUE ? TRUE : FALSE;
+  _COGL_GET_CONTEXT (ctx, FALSE);
+  return ctx->legacy_depth_test_enabled;
 }
 
 void
@@ -389,7 +388,7 @@ cogl_set_viewport (int x,
                    int width,
                    int height)
 {
-  CoglHandle framebuffer;
+  CoglFramebuffer *framebuffer;
 
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
@@ -493,6 +492,9 @@ cogl_get_features (void)
   if (G_UNLIKELY (cogl_debug_flags & COGL_DEBUG_DISABLE_VBOS))
     ctx->feature_flags &= ~COGL_FEATURE_VBOS;
 
+  if (G_UNLIKELY (cogl_debug_flags & COGL_DEBUG_DISABLE_GLSL))
+    ctx->feature_flags &= ~COGL_FEATURE_SHADERS_GLSL;
+
   return ctx->feature_flags;
 }
 
@@ -507,6 +509,20 @@ cogl_features_available (CoglFeatureFlags features)
   return (ctx->feature_flags & features) == features;
 }
 
+gboolean
+_cogl_features_available_private (CoglFeatureFlagsPrivate features)
+{
+  _COGL_GET_CONTEXT (ctx, 0);
+
+  if (!ctx->features_cached)
+    _cogl_features_init ();
+
+  if (G_UNLIKELY (cogl_debug_flags & COGL_DEBUG_DISABLE_ARBFP))
+    ctx->feature_flags_private &= ~COGL_FEATURE_PRIVATE_ARB_FP;
+
+  return (ctx->feature_flags_private & features) == features;
+}
+
 /* XXX: This function should either be replaced with one returning
  * integers, or removed/deprecated and make the
  * _cogl_framebuffer_get_viewport* functions public.
@@ -514,7 +530,7 @@ cogl_features_available (CoglFeatureFlags features)
 void
 cogl_get_viewport (float v[4])
 {
-  CoglHandle framebuffer;
+  CoglFramebuffer *framebuffer;
   int viewport[4];
   int i;
 
@@ -538,16 +554,16 @@ cogl_get_bitmasks (int *red,
   framebuffer = _cogl_get_framebuffer ();
 
   if (red)
-    *red = cogl_framebuffer_get_red_bits (framebuffer);
+    *red = _cogl_framebuffer_get_red_bits (framebuffer);
 
   if (green)
-    *green = cogl_framebuffer_get_green_bits (framebuffer);
+    *green = _cogl_framebuffer_get_green_bits (framebuffer);
 
   if (blue)
-    *blue = cogl_framebuffer_get_blue_bits (framebuffer);
+    *blue = _cogl_framebuffer_get_blue_bits (framebuffer);
 
   if (alpha)
-    *alpha = cogl_framebuffer_get_alpha_bits (framebuffer);
+    *alpha = _cogl_framebuffer_get_alpha_bits (framebuffer);
 }
 
 void
@@ -560,6 +576,8 @@ cogl_set_fog (const CoglColor *fog_color,
   GLfloat fogColor[4];
   GLenum gl_mode = GL_LINEAR;
 
+  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
+
   /* The cogl journal doesn't currently track fog state changes */
   _cogl_journal_flush ();
 
@@ -595,15 +613,20 @@ cogl_set_fog (const CoglColor *fog_color,
   glFogf (GL_FOG_DENSITY, (GLfloat) density);
   glFogf (GL_FOG_START, (GLfloat) z_near);
   glFogf (GL_FOG_END, (GLfloat) z_far);
+
+  ctx->fog_enabled = TRUE;
 }
 
 void
 cogl_disable_fog (void)
 {
+  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
+
   /* Currently the journal can't track changes to fog state... */
   _cogl_journal_flush ();
 
   glDisable (GL_FOG);
+  ctx->fog_enabled = FALSE;
 }
 
 void
@@ -621,7 +644,7 @@ cogl_read_pixels (int x,
                   CoglPixelFormat format,
                   guint8 *pixels)
 {
-  CoglHandle framebuffer;
+  CoglFramebuffer *framebuffer;
   int        framebuffer_height;
   int        bpp;
   CoglBitmap bmp;
@@ -790,12 +813,12 @@ _cogl_disable_other_texcoord_arrays (const CoglBitmask *mask)
   /* Update the mask of arrays that are enabled */
   _cogl_bitmask_clear_bits (&ctx->texcoord_arrays_enabled,
                             &ctx->texcoord_arrays_to_disable);
+  _cogl_bitmask_set_bits (&ctx->texcoord_arrays_enabled, mask);
 }
 
 void
 cogl_begin_gl (void)
 {
-  CoglMaterialFlushOptions options;
   unsigned long enable_flags = 0;
 
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
@@ -830,12 +853,7 @@ cogl_begin_gl (void)
    * A user should instead call cogl_set_source_color4ub() before
    * cogl_begin_gl() to simplify the state flushed.
    */
-  options.flags = 0;
-  _cogl_material_flush_gl_state (ctx->source_material, &options);
-
-  /* FIXME: This api is a bit yukky, ideally it will be removed if we
-   * re-work the _cogl_enable mechanism */
-  enable_flags |= _cogl_material_get_cogl_enable_flags (ctx->source_material);
+  _cogl_material_flush_gl_state (ctx->source_material, FALSE);
 
   if (ctx->enable_backface_culling)
     enable_flags |= COGL_ENABLE_BACKFACE_CULLING;
@@ -864,102 +882,6 @@ cogl_end_gl (void)
   ctx->in_begin_gl_block = FALSE;
 }
 
-static CoglTextureUnit *
-_cogl_texture_unit_new (int index_)
-{
-  CoglTextureUnit *unit = g_new0 (CoglTextureUnit, 1);
-  unit->matrix_stack = _cogl_matrix_stack_new ();
-  unit->index = index_;
-  return unit;
-}
-
-static void
-_cogl_texture_unit_free (CoglTextureUnit *unit)
-{
-  _cogl_matrix_stack_destroy (unit->matrix_stack);
-  g_free (unit);
-}
-
-CoglTextureUnit *
-_cogl_get_texture_unit (int index_)
-{
-  GList *l;
-  CoglTextureUnit *unit;
-
-  _COGL_GET_CONTEXT (ctx, NULL);
-
-  for (l = ctx->texture_units; l; l = l->next)
-    {
-      unit = l->data;
-
-      if (unit->index == index_)
-        return unit;
-
-      /* The units are always sorted, so at this point we know this unit
-       * doesn't exist */
-      if (unit->index > index_)
-        break;
-    }
-  /* NB: if we now insert a new layer before l, that will maintain order.
-   */
-
-  unit = _cogl_texture_unit_new (index_);
-
-  /* Note: see comment after for() loop above */
-  ctx->texture_units =
-    g_list_insert_before (ctx->texture_units, l, unit);
-
-  return unit;
-}
-
-void
-_cogl_destroy_texture_units (void)
-{
-  GList *l;
-
-  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
-
-  for (l = ctx->texture_units; l; l = l->next)
-    _cogl_texture_unit_free (l->data);
-  g_list_free (ctx->texture_units);
-}
-
-/*
- * This is more complicated than that, another pass needs to be done when
- * cogl have a neat way of saying if we are using the fixed function pipeline
- * or not (for the GL case):
- * MAX_TEXTURE_UNITS: fixed function pipeline, a texture unit has both a
- *                    sampler and a set of texture coordinates
- * MAX_TEXTURE_IMAGE_UNITS: number of samplers one can use from a fragment
- *                          program/shader (ARBfp1.0 asm/GLSL)
- * MAX_VERTEX_TEXTURE_UNITS: number of samplers one can use from a vertex
- *                           program/shader (can be 0)
- * MAX_COMBINED_TEXTURE_IMAGE_UNITS: Maximum samplers one can use, counting both
- *                                   the vertex and fragment shaders
- *
- * If both the vertex shader and the fragment processing stage access the same
- * texture image unit, then that counts as using two texture image units
- * against the latter limit: http://www.opengl.org/sdk/docs/man/xhtml/glGet.xml
- *
- * Note that, for now, we use GL_MAX_TEXTURE_UNITS as we are exposing the
- * fixed function pipeline.
- */
-unsigned int
-_cogl_get_max_texture_image_units (void)
-{
-  _COGL_GET_CONTEXT (ctx, 0);
-
-  /* This function is called quite often so we cache the value to
-     avoid too many GL calls */
-  if (ctx->max_texture_units == -1)
-    {
-      ctx->max_texture_units = 1;
-      GE( glGetIntegerv (GL_MAX_TEXTURE_UNITS, &ctx->max_texture_units) );
-    }
-
-  return ctx->max_texture_units;
-}
-
 void
 cogl_push_matrix (void)
 {
@@ -1109,7 +1031,7 @@ cogl_set_projection_matrix (CoglMatrix *matrix)
 CoglClipState *
 _cogl_get_clip_state (void)
 {
-  CoglHandle framebuffer;
+  CoglFramebuffer *framebuffer;
 
   framebuffer = _cogl_get_framebuffer ();
   return _cogl_framebuffer_get_clip_state (framebuffer);
@@ -1146,7 +1068,7 @@ cogl_set_source_texture (CoglHandle texture_handle)
 
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
-  g_return_if_fail (texture_handle != COGL_INVALID_HANDLE);
+  g_return_if_fail (texture_handle != NULL);
 
   cogl_material_set_layer (ctx->simple_material, 0, texture_handle);
   cogl_color_set_from_4ub (&white, 0xff, 0xff, 0xff, 0xff);
@@ -1178,3 +1100,49 @@ cogl_set_source_color4f (float red,
   cogl_set_source_color (&c);
 }
 
+/* Scale from OpenGL normalized device coordinates (ranging from -1 to 1)
+ * to Cogl window/framebuffer coordinates (ranging from 0 to buffer-size) with
+ * (0,0) being top left. */
+#define VIEWPORT_TRANSFORM_X(x, vp_origin_x, vp_width) \
+    (  ( ((x) + 1.0) * ((vp_width) / 2.0) ) + (vp_origin_x)  )
+/* Note: for Y we first flip all coordinates around the X axis while in
+ * normalized device coodinates */
+#define VIEWPORT_TRANSFORM_Y(y, vp_origin_y, vp_height) \
+    (  ( ((-(y)) + 1.0) * ((vp_height) / 2.0) ) + (vp_origin_y)  )
+
+/* Transform a homogeneous vertex position from model space to Cogl
+ * window coordinates (with 0,0 being top left) */
+void
+_cogl_transform_point (const CoglMatrix *matrix_mv,
+                       const CoglMatrix *matrix_p,
+                       const float *viewport,
+                       float *x,
+                       float *y)
+{
+  float z = 0;
+  float w = 1;
+
+  /* Apply the modelview matrix transform */
+  cogl_matrix_transform_point (matrix_mv, x, y, &z, &w);
+
+  /* Apply the projection matrix transform */
+  cogl_matrix_transform_point (matrix_p, x, y, &z, &w);
+
+  /* Perform perspective division */
+  *x /= w;
+  *y /= w;
+
+  /* Apply viewport transform */
+  *x = VIEWPORT_TRANSFORM_X (*x, viewport[0], viewport[2]);
+  *y = VIEWPORT_TRANSFORM_Y (*y, viewport[1], viewport[3]);
+}
+
+#undef VIEWPORT_TRANSFORM_X
+#undef VIEWPORT_TRANSFORM_Y
+
+GQuark
+_cogl_error_quark (void)
+{
+  return g_quark_from_static_string ("cogl-error-quark");
+}
+
