@@ -162,8 +162,8 @@ cogl_get_bitmasks (int *red,
                    int *blue,
                    int *alpha);
 
-/**
- * cogl_framebuffer_get_red_bits:
+/*
+ * _cogl_framebuffer_get_red_bits:
  * @framebuffer: a handle for a framebuffer
  *
  * Retrieves the number of red bits of @framebuffer
@@ -173,10 +173,10 @@ cogl_get_bitmasks (int *red,
  * Since: 1.4
  */
 int
-cogl_framebuffer_get_red_bits (CoglHandle framebuffer);
+_cogl_framebuffer_get_red_bits (CoglHandle framebuffer);
 
-/**
- * cogl_framebuffer_get_green_bits:
+/*
+ * _cogl_framebuffer_get_green_bits:
  * @framebuffer: a handle for a framebuffer
  *
  * Retrieves the number of green bits of @framebuffer
@@ -186,10 +186,10 @@ cogl_framebuffer_get_red_bits (CoglHandle framebuffer);
  * Since: 1.4
  */
 int
-cogl_framebuffer_get_green_bits (CoglHandle framebuffer);
+_cogl_framebuffer_get_green_bits (CoglHandle framebuffer);
 
-/**
- * cogl_framebuffer_get_blue_bits:
+/*
+ * _cogl_framebuffer_get_blue_bits:
  * @framebuffer: a handle for a framebuffer
  *
  * Retrieves the number of blue bits of @framebuffer
@@ -199,10 +199,10 @@ cogl_framebuffer_get_green_bits (CoglHandle framebuffer);
  * Since: 1.4
  */
 int
-cogl_framebuffer_get_blue_bits (CoglHandle framebuffer);
+_cogl_framebuffer_get_blue_bits (CoglHandle framebuffer);
 
-/**
- * cogl_framebuffer_get_alpha_bits:
+/*
+ * _cogl_framebuffer_get_alpha_bits:
  * @framebuffer: a handle for a framebuffer
  *
  * Retrieves the number of alpha bits of @framebuffer
@@ -212,7 +212,7 @@ cogl_framebuffer_get_blue_bits (CoglHandle framebuffer);
  * Since: 1.4
  */
 int
-cogl_framebuffer_get_alpha_bits (CoglHandle framebuffer);
+_cogl_framebuffer_get_alpha_bits (CoglHandle framebuffer);
 
 /**
  * cogl_perspective:
@@ -258,13 +258,26 @@ cogl_frustum (float left,
  * @right: The coordinate for the right clipping plane
  * @bottom: The coordinate for the bottom clipping plane
  * @top: The coordinate for the top clipping plane
- * @near: The coordinate for the near clipping plane (may be negative if
+ * @near: The <b>distance</b> to the near clipping plane (negative if
  *        the plane is behind the viewer)
- * @far: The coordinate for the far clipping plane (may be negative if
+ * @far: The <b>distance</b> for the far clipping plane (negative if
  *       the plane is behind the viewer)
  *
- * Replaces the current projection matrix with a parallel projection
- * matrix.
+ * Replaces the current projection matrix with an orthographic projection
+ * matrix. See <xref linkend="cogl-ortho-matrix"/> to see how the matrix is
+ * calculated.
+ *
+ * <figure id="cogl-ortho-matrix">
+ *   <title></title>
+ *   <graphic fileref="cogl_ortho.png" format="PNG"/>
+ * </figure>
+ *
+ * <note>cogl_ortho copies the arguments from OpenGL's glOrtho even
+ * though they are unnecessarily confusing due to the z near and z far
+ * arguments actually being a "distance" from the origin, where
+ * negative values are behind the viewer, instead of coordinates for
+ * the z clipping planes which would have been consistent with the
+ * left, right bottom and top arguments.</note>
  *
  * Since: 1.0
  */
@@ -470,6 +483,9 @@ cogl_get_viewport (float v[4]);
  * order specified using clutter_actor_raise() and
  * clutter_actor_lower(), otherwise it will also take into account the
  * actor's depth. Depth testing is disabled by default.
+ *
+ * Deprecated: 1.4: Use cogl_material_set_depth_test_enabled()
+ * instead.
  */
 void
 cogl_set_depth_test_enabled (gboolean setting);
@@ -480,6 +496,9 @@ cogl_set_depth_test_enabled (gboolean setting);
  * Queries if depth testing has been enabled via cogl_set_depth_test_enable()
  *
  * Return value: %TRUE if depth testing is enabled, and %FALSE otherwise
+ *
+ * Deprecated: 1.4: Use cogl_material_get_depth_test_enabled()
+ * instead.
  */
 gboolean
 cogl_get_depth_test_enabled (void);
