@@ -40,6 +40,7 @@
 #include "cogl-context.h"
 #include "cogl-handle.h"
 #include "cogl-primitives.h"
+#include "cogl-material-private.h"
 
 #include <string.h>
 #include <stdlib.h>
@@ -58,7 +59,9 @@ _cogl_texture_driver_gen (GLenum   gl_target,
 
   for (i = 0; i < n; i++)
     {
-      GE (glBindTexture (gl_target, textures[i]));
+      _cogl_bind_gl_texture_transient (gl_target,
+                                       textures[i],
+                                       FALSE);
 
       switch (gl_target)
         {
@@ -67,6 +70,11 @@ _cogl_texture_driver_gen (GLenum   gl_target,
         GE( glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR) );
         break;
 
+      case GL_TEXTURE_RECTANGLE_ARB:
+        /* Texture rectangles already default to GL_LINEAR so nothing
+           needs to be done */
+        break;
+
       default:
         g_assert_not_reached();
         }
@@ -123,6 +131,7 @@ _cogl_texture_driver_prep_gl_for_pixels_download (int pixels_rowstride,
 void
 _cogl_texture_driver_upload_subregion_to_gl (GLenum       gl_target,
                                              GLuint       gl_handle,
+                                             gboolean     is_foreign,
                                              int          src_x,
                                              int          src_y,
                                              int          dst_x,
@@ -141,7 +150,7 @@ _cogl_texture_driver_upload_subregion_to_gl (GLenum       gl_target,
                                   src_y,
                                   bpp);
 
-  GE( glBindTexture (gl_target, gl_handle) );
+  _cogl_bind_gl_texture_transient (gl_target, gl_handle, is_foreign);
 
   GE( glTexSubImage2D (gl_target, 0,
                        dst_x, dst_y,
@@ -154,6 +163,7 @@ _cogl_texture_driver_upload_subregion_to_gl (GLenum       gl_target,
 void
 _cogl_texture_driver_upload_to_gl (GLenum       gl_target,
                                    GLuint       gl_handle,
+                                   gboolean     is_foreign,
                                    CoglBitmap  *source_bmp,
                                    GLint        internal_gl_format,
                                    GLuint       source_gl_format,
@@ -164,7 +174,7 @@ _cogl_texture_driver_upload_to_gl (GLenum       gl_target,
   /* Setup gl alignment to match rowstride and top-left corner */
   prep_gl_for_pixels_upload_full (source_bmp->rowstride, 0, 0, bpp);
 
-  GE( glBindTexture (gl_target, gl_handle) );
+  _cogl_bind_gl_texture_transient (gl_target, gl_handle, is_foreign);
 
   GE( glTexImage2D (gl_target, 0,
                     internal_gl_format,
@@ -196,26 +206,28 @@ _cogl_texture_driver_size_supported (GLenum gl_target,
                                      int    width,
                                      int    height)
 {
-  if (gl_target ==  GL_TEXTURE_2D)
-    {
-      /* Proxy texture allows for a quick check for supported size */
+  GLenum proxy_target;
+  GLint new_width = 0;
 
-      GLint new_width = 0;
-
-      GE( glTexImage2D (GL_PROXY_TEXTURE_2D, 0, GL_RGBA,
-			width, height, 0 /* border */,
-			gl_format, gl_type, NULL) );
-
-      GE( glGetTexLevelParameteriv (GL_PROXY_TEXTURE_2D, 0,
-				    GL_TEXTURE_WIDTH, &new_width) );
-
-      return new_width != 0;
-    }
+  if (gl_target == GL_TEXTURE_2D)
+    proxy_target = GL_PROXY_TEXTURE_2D;
+#if HAVE_COGL_GL
+  else if (gl_target == GL_TEXTURE_RECTANGLE_ARB)
+    proxy_target = GL_PROXY_TEXTURE_RECTANGLE_ARB;
+#endif
   else
-    {
-      /* not used */
-      return 0;
-    }
+    /* Unknown target, assume it's not supported */
+    return FALSE;
+
+  /* Proxy texture allows for a quick check for supported size */
+  GE( glTexImage2D (proxy_target, 0, GL_RGBA,
+                    width, height, 0 /* border */,
+                    gl_format, gl_type, NULL) );
+
+  GE( glGetTexLevelParameteriv (proxy_target, 0,
+                                GL_TEXTURE_WIDTH, &new_width) );
+
+  return new_width != 0;
 }
 
 void
