@@ -18,9 +18,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
+ *
+ *
  */
 
 /**
@@ -43,9 +43,4 @@
 #include <clutter/clutter.h>
 #include <clutter/glx/clutter-glx-texture-pixmap.h>
 
-G_BEGIN_DECLS
-
-
-G_END_DECLS
-
 #endif /* __CLUTTER_GLX_H__ */
