@@ -14,9 +14,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
+ *
+ *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -28,6 +28,7 @@
 #include "clutter-backend-win32.h"
 #include "clutter-stage-win32.h"
 #include "clutter-win32.h"
+#include "clutter-device-manager-win32.h"
 
 #include "../clutter-event.h"
 #include "../clutter-main.h"
@@ -66,28 +67,13 @@ static void
 clutter_backend_win32_init_events (ClutterBackend *backend)
 {
   ClutterBackendWin32 *backend_win32 = CLUTTER_BACKEND_WIN32 (backend);
-  ClutterDeviceManager *manager;
-  ClutterInputDevice *device;
 
   CLUTTER_NOTE (EVENT, "initialising the event loop");
 
-  manager = clutter_device_manager_get_default ();
-
-  device = g_object_new (CLUTTER_TYPE_INPUT_DEVICE,
-                         "id", 0,
-                         "name", "Core Pointer",
-                         "device-type", CLUTTER_POINTER_DEVICE,
-                         NULL);
-  _clutter_device_manager_add_device (manager, device);
-  backend_win32->core_pointer = device;
-
-  device = g_object_new (CLUTTER_TYPE_INPUT_DEVICE,
-                         "id", 1,
-                         "name", "Core Keyboard",
-                         "device-type", CLUTTER_KEYBOARD_DEVICE,
-                         NULL);
-  _clutter_device_manager_add_device (manager, device);
-  backend_win32->core_keyboard = device;
+  backend_win32->device_manager =
+    g_object_new (CLUTTER_TYPE_DEVICE_MANAGER_WIN32,
+                  "backend", backend_win32,
+                  NULL);
 
   _clutter_backend_win32_events_init (backend);
 }
@@ -242,7 +228,7 @@ clutter_backend_win32_get_features (ClutterBackend *backend)
      for the swap control extension */
   if (getenv ("__GL_SYNC_TO_VBLANK") || check_vblank_env ("default"))
     CLUTTER_NOTE (BACKEND, "vblank sync: left at default at user request");
-  else if (cogl_check_extension ("WGL_EXT_swap_control", extensions)
+  else if (_cogl_check_extension ("WGL_EXT_swap_control", extensions)
 	   && (swap_interval = (SwapIntervalProc)
 	       cogl_get_proc_address ("wglSwapIntervalEXT")))
     {
@@ -537,6 +523,22 @@ clutter_backend_win32_create_stage (ClutterBackend  *backend,
   return stage;
 }
 
+static ClutterDeviceManager *
+clutter_backend_win32_get_device_manager (ClutterBackend *backend)
+{
+  ClutterBackendWin32 *backend_win32 = CLUTTER_BACKEND_WIN32 (backend);
+
+  if (G_UNLIKELY (backend_win32->device_manager == NULL))
+    {
+      backend_win32->device_manager =
+        g_object_new (CLUTTER_TYPE_DEVICE_MANAGER_WIN32,
+                      "backend", backend_win32,
+                      NULL);
+    }
+
+  return backend_win32->device_manager;
+}
+
 static void
 clutter_backend_win32_class_init (ClutterBackendWin32Class *klass)
 {
@@ -555,6 +557,7 @@ clutter_backend_win32_class_init (ClutterBackendWin32Class *klass)
   backend_class->redraw           = clutter_backend_win32_redraw;
   backend_class->create_context   = clutter_backend_win32_create_context;
   backend_class->ensure_context   = clutter_backend_win32_ensure_context;
+  backend_class->get_device_manager = clutter_backend_win32_get_device_manager;
 }
 
 static void
@@ -566,10 +569,11 @@ clutter_backend_win32_init (ClutterBackendWin32 *backend_win32)
   backend_win32->no_event_retrieval = FALSE;
   backend_win32->invisible_cursor   = NULL;
 
-  /* FIXME: get from GetSystemMetric? */
+  /* FIXME: get from GetSystemMetric?
   clutter_backend_set_double_click_time (backend, 250);
   clutter_backend_set_double_click_distance (backend, 5);
   clutter_backend_set_resolution (backend, 96.0);
+  */
 
   /* Set the maximum precision for Windows time functions. Without
      this glib will not be able to sleep accurately enough to give a
